@@ -33,6 +33,7 @@
 #include <fstream>
 
 #include <test/support/tdb_catch.h>
+#include "tiledb/api/c_api/enumeration/enumeration_api_internal.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
@@ -47,7 +48,7 @@ struct CPPEnumerationFx {
   template <typename T>
   void check_dump(const T& val);
 
-  void create_array();
+  void create_array(bool with_empty_enumeration = false);
   void rm_array();
 
   std::string uri_;
@@ -122,6 +123,112 @@ TEST_CASE_METHOD(
   REQUIRE(data == values);
 }
 
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Extend Fixed Size",
+    "[enumeration][extend][fixed-size]") {
+  std::vector<int> init_values = {1, 2, 3, 4, 5};
+  std::vector<int> add_values = {6, 7, 8, 9, 10};
+  std::vector<int> final_values = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+  auto enmr1 = Enumeration::create(ctx_, enmr_name, init_values, true);
+  auto enmr2 = enmr1.extend(add_values);
+
+  REQUIRE(enmr2.ptr()->is_extension_of(enmr1.ptr().get()));
+
+  auto data = enmr2.as_vector<int>();
+  REQUIRE(data == final_values);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Extend Var Size",
+    "[enumeration][extend][var-size]") {
+  std::vector<std::string> init_values = {"fred", "wilma"};
+  std::vector<std::string> add_values = {"barney", "betty"};
+  std::vector<std::string> final_values = {"fred", "wilma", "barney", "betty"};
+  auto enmr1 = Enumeration::create(ctx_, enmr_name, init_values, true);
+  auto enmr2 = enmr1.extend(add_values);
+
+  REQUIRE(enmr2.ptr()->is_extension_of(enmr1.ptr().get()));
+
+  auto data = enmr2.as_vector<std::string>();
+  REQUIRE(data == final_values);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Fixed Size Empty Vector extension",
+    "[enumeration][extend][error]") {
+  std::vector<int> init_values = {1, 2, 3, 4, 5};
+  std::vector<int> add_values = {};
+  auto enmr = Enumeration::create(ctx_, enmr_name, init_values, true);
+
+  auto matcher = Catch::Matchers::ContainsSubstring(
+      "Unable to extend an enumeration with an empty vector.");
+  REQUIRE_THROWS_WITH(enmr.extend(add_values), matcher);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Var Size Empty Vector extension",
+    "[enumeration][extend][error]") {
+  std::vector<std::string> init_values = {"fred", "wilma"};
+  std::vector<std::string> add_values = {};
+  auto enmr = Enumeration::create(ctx_, enmr_name, init_values, true);
+
+  auto matcher = Catch::Matchers::ContainsSubstring(
+      "Unable to extend an enumeration with an empty vector.");
+  REQUIRE_THROWS_WITH(enmr.extend(add_values), matcher);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Invalid cell val num extension",
+    "[enumeration][extend][error]") {
+  std::vector<int> init_values = {1, 2, 3, 4};
+  std::vector<int> add_values = {5};
+  auto enmr = Enumeration::create(
+      ctx_,
+      enmr_name,
+      TILEDB_INT32,
+      2,
+      true,
+      init_values.data(),
+      init_values.size() * sizeof(int),
+      nullptr,
+      0);
+
+  auto matcher = Catch::Matchers::ContainsSubstring(
+      "Invalid data size is not a multiple of the cell size.");
+  REQUIRE_THROWS_WITH(enmr.extend(add_values), matcher);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Fixed Size Wrong Type extension",
+    "[enumeration][extend][error]") {
+  std::vector<int> init_values = {1, 2, 3, 4, 5};
+  std::vector<std::string> add_values = {"barney", "betty"};
+  auto enmr = Enumeration::create(ctx_, enmr_name, init_values, true);
+
+  auto matcher = Catch::Matchers::ContainsSubstring(
+      "Error extending fixed sized enumeration with var size data.");
+  REQUIRE_THROWS_WITH(enmr.extend(add_values), matcher);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration API - Var Size Wrong Type extension",
+    "[enumeration][extend][error]") {
+  std::vector<std::string> init_values = {"fred", "wilma"};
+  std::vector<int> add_values = {6, 7, 8, 9, 10};
+  auto enmr = Enumeration::create(ctx_, enmr_name, init_values, true);
+
+  auto matcher = Catch::Matchers::ContainsSubstring(
+      "Error extending var sized enumeration with fixed size data.");
+  REQUIRE_THROWS_WITH(enmr.extend(add_values), matcher);
+}
+
 TEST_CASE_METHOD(
     CPPEnumerationFx,
     "CPP: Enumeration API - Dump Basic",
@@ -239,6 +346,33 @@ TEST_CASE_METHOD(
   REQUIRE(rc != TILEDB_OK);
 }
 
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: ArraySchemaEvolution - Extend Enumeration",
+    "[enumeration][array-schema-evolution][extend-enumeration]") {
+  ArraySchemaEvolution ase(ctx_);
+  std::vector<std::string> values = {"fred", "wilma", "barney", "pebbles"};
+  auto enmr = Enumeration::create(ctx_, enmr_name, values);
+  CHECK_NOTHROW(ase.extend_enumeration(enmr));
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "C API: ArraySchemaEvolution - Extend Enumeration - Check nullptr",
+    "[enumeration][array-schema-evolution][drop-enumeration]") {
+  std::vector<std::string> values = {"fred", "wilma", "barney", "pebbles"};
+  auto enmr = Enumeration::create(ctx_, enmr_name, values);
+
+  auto rc = tiledb_array_schema_evolution_extend_enumeration(
+      ctx_.ptr().get(), nullptr, enmr.ptr().get());
+  REQUIRE(rc != TILEDB_OK);
+
+  ArraySchemaEvolution ase(ctx_);
+  rc = tiledb_array_schema_evolution_extend_enumeration(
+      ctx_.ptr().get(), ase.ptr().get(), nullptr);
+  REQUIRE(rc != TILEDB_OK);
+}
+
 TEST_CASE_METHOD(
     CPPEnumerationFx,
     "CPP: ArraySchemaEvolution - Drop Enumeration",
@@ -279,7 +413,9 @@ TEST_CASE_METHOD(
 
   auto array = Array(ctx_, uri_, TILEDB_READ);
   Query query(ctx_, array);
-  query.add_range("dim", 1, 5)
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("dim", dim)
       .set_data_buffer("attr1", attr1)
@@ -315,7 +451,9 @@ TEST_CASE_METHOD(
 
   auto array = Array(ctx_, uri_, TILEDB_READ);
   Query query(ctx_, array);
-  query.add_range("dim", 1, 5)
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("dim", dim)
       .set_data_buffer("attr1", attr1)
@@ -356,7 +494,9 @@ TEST_CASE_METHOD(
 
   auto array = Array(ctx_, uri_, TILEDB_READ);
   Query query(ctx_, array);
-  query.add_range("dim", 1, 5)
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("dim", dim)
       .set_data_buffer("attr1", attr1)
@@ -388,13 +528,13 @@ TEST_CASE_METHOD(
 
 TEST_CASE_METHOD(
     CPPEnumerationFx,
-    "CPP: Enumeration Query - Invalid Enumeration Value",
+    "CPP: Enumeration Query - Invalid Enumeration Value is Always False",
     "[enumeration][query][basic]") {
   create_array();
 
   // Attempt to query with an enumeration value that isn't in the Enumeration
   QueryCondition qc(ctx_);
-  qc.init("attr1", "alf", 4, TILEDB_EQ);
+  qc.init("attr1", "alf", 3, TILEDB_EQ);
 
   // Execute the query condition against the array
   std::vector<int> dim(5);
@@ -402,16 +542,76 @@ TEST_CASE_METHOD(
 
   auto array = Array(ctx_, uri_, TILEDB_READ);
   Query query(ctx_, array);
-  query.add_range("dim", 1, 5)
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("dim", dim)
       .set_data_buffer("attr1", attr1)
       .set_condition(qc);
 
-  // Check that the error message is helpful to users.
-  auto matcher = Catch::Matchers::ContainsSubstring(
-      "Enumeration value not found for field 'attr1'");
-  REQUIRE_THROWS_WITH(query.submit(), matcher);
+  REQUIRE_NOTHROW(query.submit());
+
+  std::vector<int> dim_expect = {1, 2, 3, 4, 5};
+  std::vector<int> attr1_expect = {
+      INT32_MIN, INT32_MIN, INT32_MIN, INT32_MIN, INT32_MIN};
+
+  REQUIRE(dim == dim_expect);
+  REQUIRE(attr1 == attr1_expect);
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration Query - Invalid Enumeration Value Accepted by EQ",
+    "[enumeration][query][basic]") {
+  create_array();
+
+  // Attempt to query with an enumeration value that isn't in the Enumeration
+  QueryCondition qc(ctx_);
+  qc.init("attr1", "alf", 3, TILEDB_EQ);
+
+  // Execute the query condition against the array
+  std::vector<int> dim(5);
+  std::vector<int> attr1(5);
+
+  auto array = Array(ctx_, uri_, TILEDB_READ);
+  Query query(ctx_, array);
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
+      .set_layout(TILEDB_ROW_MAJOR)
+      .set_data_buffer("dim", dim)
+      .set_data_buffer("attr1", attr1)
+      .set_condition(qc);
+
+  CHECK_NOTHROW(query.submit());
+}
+
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration Query - Invalid Enumeration Value Accepted by IN",
+    "[enumeration][query][basic]") {
+  create_array();
+
+  // Attempt to query with an enumeration value that isn't in the Enumeration
+  std::vector<std::string> vals = {"alf", "fred"};
+  auto qc = QueryConditionExperimental::create(ctx_, "attr1", vals, TILEDB_IN);
+
+  // Execute the query condition against the array
+  std::vector<int> dim(5);
+  std::vector<int> attr1(5);
+
+  auto array = Array(ctx_, uri_, TILEDB_READ);
+  Query query(ctx_, array);
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
+      .set_layout(TILEDB_ROW_MAJOR)
+      .set_data_buffer("dim", dim)
+      .set_data_buffer("attr1", attr1)
+      .set_condition(qc);
+
+  CHECK_NOTHROW(query.submit());
 }
 
 TEST_CASE_METHOD(
@@ -435,6 +635,33 @@ TEST_CASE_METHOD(
   REQUIRE(rc != TILEDB_OK);
 }
 
+TEST_CASE_METHOD(
+    CPPEnumerationFx,
+    "CPP: Enumeration Query - Attempt to query on empty enumeration",
+    "[enumeration][query][empty-results]") {
+  create_array(true);
+
+  // Attempt to query with an enumeration value that isn't in the Enumeration
+  QueryCondition qc(ctx_);
+  qc.init("attr3", "alf", 3, TILEDB_EQ);
+
+  // Execute the query condition against the array
+  std::vector<int> dim(5);
+  std::vector<int> attr3(5);
+
+  auto array = Array(ctx_, uri_, TILEDB_READ);
+  Query query(ctx_, array);
+  Subarray subarray(ctx_, array);
+  subarray.add_range("dim", 1, 5);
+  query.set_subarray(subarray)
+      .set_layout(TILEDB_ROW_MAJOR)
+      .set_data_buffer("dim", dim)
+      .set_data_buffer("attr3", attr3)
+      .set_condition(qc);
+
+  REQUIRE_NOTHROW(query.submit());
+}
+
 CPPEnumerationFx::CPPEnumerationFx()
     : uri_("enumeration_test_array")
     , vfs_(ctx_) {
@@ -470,7 +697,7 @@ void CPPEnumerationFx::check_dump(const T& val) {
   vfs_.remove_file(dump_name);
 }
 
-void CPPEnumerationFx::create_array() {
+void CPPEnumerationFx::create_array(bool with_empty_enumeration) {
   // Create a simple array for testing. This ends up with just five elements in
   // the array. dim is an int32_t dimension, attr1 is an enumeration with string
   // values and int32_t attribute values. attr2 is a float attribute.
@@ -500,11 +727,22 @@ void CPPEnumerationFx::create_array() {
   auto attr2 = Attribute::create<float>(ctx_, "attr2");
   schema.add_attribute(attr2);
 
+  if (with_empty_enumeration) {
+    auto empty_enmr = Enumeration::create_empty(
+        ctx_, "empty_enmr", TILEDB_STRING_ASCII, TILEDB_VAR_NUM);
+    ArraySchemaExperimental::add_enumeration(ctx_, schema, empty_enmr);
+
+    auto attr3 = Attribute::create<int>(ctx_, "attr3");
+    AttributeExperimental::set_enumeration_name(ctx_, attr3, "empty_enmr");
+    schema.add_attribute(attr3);
+  }
+
   Array::create(uri_, schema);
 
   // Attribute data
   std::vector<int> attr1_values = {0, 1, 2, 1, 0};
   std::vector<float> attr2_values = {1.0f, 2.0f, 3.0f, 4.0f, 5.0f};
+  std::vector<int> attr3_values = {0, 0, 0, 0, 0};
 
   Array array(ctx_, uri_, TILEDB_WRITE);
   Subarray subarray(ctx_, array);
@@ -514,6 +752,11 @@ void CPPEnumerationFx::create_array() {
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("attr1", attr1_values)
       .set_data_buffer("attr2", attr2_values);
+
+  if (with_empty_enumeration) {
+    query.set_data_buffer("attr3", attr3_values);
+  }
+
   CHECK_NOTHROW(query.submit());
   query.finalize();
   array.close();
