@@ -32,8 +32,11 @@
 
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
+#include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/cpp_api/tiledb"
 
+using namespace tiledb;
+
 static void check_filters(
     const tiledb::FilterList& answer, const tiledb::FilterList& check) {
   REQUIRE(check.nfilters() == answer.nfilters());
@@ -45,7 +48,6 @@ static void check_filters(
 }
 
 TEST_CASE("C++ API: Filter options", "[cppapi][filter]") {
-  using namespace tiledb;
   Context ctx;
 
   // Test filter creation and option setting/getting
@@ -76,13 +78,11 @@ TEST_CASE("C++ API: Filter options", "[cppapi][filter]") {
   // Check templated version with wrong type throws exception
   uint32_t wrong_type_u = 1;
   REQUIRE_THROWS_AS(
-      f.set_option(TILEDB_COMPRESSION_LEVEL, wrong_type_u),
-      std::invalid_argument);
+      f.set_option(TILEDB_COMPRESSION_LEVEL, wrong_type_u), TileDBError);
   REQUIRE_THROWS_AS(
-      f.get_option(TILEDB_COMPRESSION_LEVEL, &wrong_type_u),
-      std::invalid_argument);
+      f.get_option(TILEDB_COMPRESSION_LEVEL, &wrong_type_u), TileDBError);
   REQUIRE_THROWS_AS(
-      f.get_option<uint32_t>(TILEDB_COMPRESSION_LEVEL), std::invalid_argument);
+      f.get_option<uint32_t>(TILEDB_COMPRESSION_LEVEL), TileDBError);
 
   // Check that you can bypass type safety (don't do this).
   f.get_option(TILEDB_COMPRESSION_LEVEL, (void*)&wrong_type_u);
@@ -101,14 +101,12 @@ TEST_CASE("C++ API: Filter options", "[cppapi][filter]") {
   int32_t wrong_type_i = 1;
   REQUIRE_THROWS_AS(f2.set_option(TILEDB_COMPRESSION_LEVEL, 1), TileDBError);
   REQUIRE_THROWS_AS(
-      f2.set_option(TILEDB_BIT_WIDTH_MAX_WINDOW, -1), std::invalid_argument);
+      f2.set_option(TILEDB_BIT_WIDTH_MAX_WINDOW, -1), TileDBError);
   REQUIRE_THROWS_AS(
-      f2.set_option(TILEDB_BIT_WIDTH_MAX_WINDOW, wrong_type_i),
-      std::invalid_argument);
+      f2.set_option(TILEDB_BIT_WIDTH_MAX_WINDOW, wrong_type_i), TileDBError);
 }
 
 TEST_CASE("C++ API: Filter lists", "[cppapi][filter]") {
-  using namespace tiledb;
   Context ctx;
 
   Filter f1(ctx, TILEDB_FILTER_BIT_WIDTH_REDUCTION),
@@ -141,14 +139,12 @@ TEST_CASE("C++ API: Filter lists", "[cppapi][filter]") {
   REQUIRE(list.nfilters() == 3);
 }
 
-TEST_CASE("C++ API: Filter lists on array", "[cppapi][filter]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
+TEST_CASE(
+    "C++ API: Filter lists on array",
+    "[cppapi][filter][rest-fails][sc-45554]") {
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
 
   // Create schema with filter lists
   FilterList a1_filters(ctx);
@@ -212,7 +208,9 @@ TEST_CASE("C++ API: Filter lists on array", "[cppapi][filter]") {
   std::string a2_read_data;
   a2_read_data.resize(7);
   Query query_r(ctx, array);
-  query_r.set_subarray(subarray)
+  Subarray sub(ctx, array);
+  sub.set_subarray(subarray);
+  query_r.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a1", a1_read)
       .set_data_buffer("a2", a2_read_data)
@@ -239,10 +237,6 @@ TEST_CASE("C++ API: Filter lists on array", "[cppapi][filter]") {
   check_filters(a1_filters, schema_r.attribute("a1").filter_list());
   check_filters(a2_filters, schema_r.attribute("a2").filter_list());
   array.close();
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 void write_sparse_array_string_attr(
@@ -250,10 +244,7 @@ void write_sparse_array_string_attr(
     const std::string& array_name,
     std::string& data,
     std::vector<uint64_t>& data_offsets,
-    tiledb_layout_t layout,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    tiledb_layout_t layout) {
   // Write to array
   std::vector<int64_t> d1 = {0, 10, 20, 20, 30, 30, 40};
   std::vector<int64_t> d2 = {0, 10, 20, 30, 30, 40, 40};
@@ -266,15 +257,12 @@ void write_sparse_array_string_attr(
   query.set_data_buffer("a1", data).set_offsets_buffer("a1", data_offsets);
 
   // Submit query
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    query.submit_and_finalize();
+  } else {
+    query.submit();
+  }
+
   array.close();
 }
 
@@ -283,10 +271,7 @@ void read_and_check_sparse_array_string_attr(
     const std::string& array_name,
     std::string& expected_data,
     std::vector<uint64_t>& expected_offsets,
-    tiledb_layout_t layout,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    tiledb_layout_t layout) {
   Array array(ctx, array_name, TILEDB_READ);
   Query query(ctx, array, TILEDB_READ);
 
@@ -299,15 +284,7 @@ void read_and_check_sparse_array_string_attr(
   query.set_offsets_buffer("a1", attr_off);
 
   // Submit query
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  query.submit();
 
   // Check the element offsets are properly returned
   CHECK(attr_val == expected_data);
@@ -318,24 +295,13 @@ void read_and_check_sparse_array_string_attr(
 
 TEST_CASE(
     "C++ API: Filter strings with RLE or Dictionary encoding, sparse array",
-    "[cppapi][filter][rle-strings][dict-strings][sparse]") {
-  bool serialized = false, refactored_query_v2 = false;
-#ifdef TILEDB_SERIALIZATION
-  serialized = GENERATE(true, false);
-  if (serialized) {
-    refactored_query_v2 = GENERATE(true, false);
-  }
-#endif
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][sparse][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
 
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create schema with filter lists
   FilterList a1_filters(ctx);
   a1_filters.add_filter({ctx, f});
@@ -369,88 +335,36 @@ TEST_CASE(
 
   SECTION("Unordered write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_UNORDERED,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
   SECTION("Global order write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_GLOBAL_ORDER,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 void write_dense_array_string_attr(
@@ -458,10 +372,7 @@ void write_dense_array_string_attr(
     const std::string& array_name,
     std::string& data,
     std::vector<uint64_t>& data_offsets,
-    tiledb_layout_t layout,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    tiledb_layout_t layout) {
   Array array(ctx, array_name, TILEDB_WRITE);
   Query query(ctx, array, TILEDB_WRITE);
 
@@ -470,15 +381,11 @@ void write_dense_array_string_attr(
   query.set_layout(layout);
   query.set_subarray(Subarray(ctx, array).set_subarray<int64_t>({0, 1, 0, 2}));
 
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    query.submit_and_finalize();
+  } else {
+    query.submit();
+  }
 
   array.close();
 }
@@ -487,10 +394,7 @@ void read_and_check_dense_array_string_attr(
     tiledb::Context ctx,
     const std::string& array_name,
     std::string& expected_data,
-    std::vector<uint64_t>& expected_offsets,
-    const bool serialized,
-    const bool refactored_query_v2) {
-  using namespace tiledb;
+    std::vector<uint64_t>& expected_offsets) {
   Array array(ctx, array_name, TILEDB_READ);
   Query query(ctx, array, TILEDB_READ);
 
@@ -503,15 +407,7 @@ void read_and_check_dense_array_string_attr(
   query.set_offsets_buffer("a1", attr_off);
 
   // Submit query
-  test::ServerQueryBuffers server_buffers_;
-  auto rc = test::submit_query_wrapper(
-      ctx,
-      array_name,
-      &query,
-      server_buffers_,
-      serialized,
-      refactored_query_v2);
-  REQUIRE(rc == TILEDB_OK);
+  query.submit();
 
   // Check the element offsets are properly returned
   CHECK(attr_val == expected_data);
@@ -522,27 +418,13 @@ void read_and_check_dense_array_string_attr(
 
 TEST_CASE(
     "C++ API: Filter strings with RLE or Dictionary encoding, dense array",
-    "[cppapi][filter][rle-strings][dict-strings][dense]") {
-  bool serialized = false, refactored_query_v2 = false;
-#ifdef TILEDB_SERIALIZATION
-  serialized = GENERATE(true, false);
-  if (serialized) {
-    refactored_query_v2 = GENERATE(true, false);
-  }
-#endif
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  // Create the array
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][dense][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
+
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create schema with filter lists
   FilterList a1_filters(ctx);
   a1_filters.add_filter({ctx, f});
@@ -576,55 +458,28 @@ TEST_CASE(
 
   SECTION("Ordered write") {
     write_dense_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_ROW_MAJOR,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     read_and_check_dense_array_string_attr(
-        ctx, array_name, a1_data, a1_offsets, serialized, refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets);
   }
   SECTION("Global order write") {
     write_dense_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_GLOBAL_ORDER,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     read_and_check_dense_array_string_attr(
-        ctx, array_name, a1_data, a1_offsets, serialized, refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets);
   }
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 TEST_CASE(
     "C++ API: Filter UTF-8 strings with RLE or Dictionary encoding, sparse "
     "array",
-    "[cppapi][filter][rle-strings][dict-strings][sparse][utf-8]") {
-  bool serialized = false, refactored_query_v2 = false;
-#ifdef TILEDB_SERIALIZATION
-  serialized = GENERATE(true, false);
-  if (serialized) {
-    refactored_query_v2 = GENERATE(true, false);
-  }
-#endif
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][sparse][utf-8][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
 
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create schema with filter lists
   FilterList a1_filters(ctx);
   a1_filters.add_filter({ctx, f});
@@ -662,104 +517,48 @@ TEST_CASE(
 
   SECTION("Unordered write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_UNORDERED,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
   SECTION("Global order write") {
     write_sparse_array_string_attr(
-        ctx,
-        array_name,
-        a1_data,
-        a1_offsets,
-        TILEDB_GLOBAL_ORDER,
-        serialized,
-        refactored_query_v2);
+        ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     SECTION("Row major read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_ROW_MAJOR,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_ROW_MAJOR);
     }
     SECTION("Global order read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_GLOBAL_ORDER,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_GLOBAL_ORDER);
     }
     SECTION("Unordered read") {
       read_and_check_sparse_array_string_attr(
-          ctx,
-          array_name,
-          a1_data,
-          a1_offsets,
-          TILEDB_UNORDERED,
-          serialized,
-          refactored_query_v2);
+          ctx, array_name, a1_data, a1_offsets, TILEDB_UNORDERED);
     }
   }
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
 
 TEST_CASE(
     "C++ API: Filter buffer with some empty strings with RLE or Dictionary "
     "encoding",
-    "[cppapi][filter][rle-strings][dict-strings][empty-strings]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-  std::string array_name = "cpp_unit_array";
-
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
-
+    "[cppapi][filter][rle-strings][dict-strings][empty-strings][rest]") {
   auto f = GENERATE(TILEDB_FILTER_RLE, TILEDB_FILTER_DICTIONARY);
 
+  test::VFSTestSetup vfs_test_setup{};
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_name{vfs_test_setup.array_uri("cpp_unit_array")};
+
   // Create array with string dimension and one attribute
   ArraySchema schema(ctx, TILEDB_SPARSE);
 
@@ -841,8 +640,4 @@ TEST_CASE(
   }
 
   array_r.close();
-
-  // Clean up
-  if (vfs.is_dir(array_name))
-    vfs.remove_dir(array_name);
 }
