@@ -32,6 +32,7 @@
 
 #include "test/support/src/helpers.h"
 #include "tiledb/common/common.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/query/query_buffer.h"
@@ -60,6 +61,7 @@ struct CSparseUnorderedWithDupsFx {
   std::string array_name_;
   const char* ARRAY_NAME = "test_sparse_unordered_with_dups";
   std::string total_budget_;
+  std::string tile_upper_memory_limit_;
   std::string ratio_tile_ranges_;
   std::string ratio_array_data_;
   std::string ratio_coords_;
@@ -165,6 +167,16 @@ void CSparseUnorderedWithDupsFx::update_config() {
       TILEDB_OK);
   REQUIRE(error == nullptr);
 
+  if (tile_upper_memory_limit_ != "") {
+    REQUIRE(
+        tiledb_config_set(
+            config,
+            "sm.mem.tile_upper_memory_limit",
+            tile_upper_memory_limit_.c_str(),
+            &error) == TILEDB_OK);
+    REQUIRE(error == nullptr);
+  }
+
   REQUIRE(
       tiledb_config_set(
           config,
@@ -409,8 +421,14 @@ int32_t CSparseUnorderedWithDupsFx::read(
   if (set_subarray) {
     // Set subarray.
     int subarray[] = {1, 200};
-    rc = tiledb_query_set_subarray(ctx_, query, subarray);
+    tiledb_subarray_t* sub;
+    rc = tiledb_subarray_alloc(ctx_, array, &sub);
     CHECK(rc == TILEDB_OK);
+    rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+    CHECK(rc == TILEDB_OK);
+    tiledb_subarray_free(&sub);
   }
 
   if (qc_idx != 0) {
@@ -674,8 +692,14 @@ void CSparseUnorderedWithDupsVarDataFx::read_and_check_data(bool set_subarray) {
   if (set_subarray) {
     // Set subarray.
     int64_t subarray[] = {1, 4, 1, 4};
-    rc = tiledb_query_set_subarray(ctx_, query, subarray);
+    tiledb_subarray_t* sub;
+    rc = tiledb_subarray_alloc(ctx_, array, &sub);
     CHECK(rc == TILEDB_OK);
+    rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+    CHECK(rc == TILEDB_OK);
+    tiledb_subarray_free(&sub);
   }
 
   std::vector<int32_t> data(3);
@@ -767,10 +791,10 @@ CSparseUnorderedWithDupsVarDataFx::open_default_array_1d_with_fragments(
   shared_ptr<FragmentMetadata> fragment = make_shared<FragmentMetadata>(
       HERE(),
       nullptr,
-      nullptr,
       array->array_->array_schema_latest_ptr(),
-      URI(),
+      generate_fragment_uri(array->array_.get()),
       std::make_pair<uint64_t, uint64_t>(0, 0),
+      tiledb::test::create_test_memory_tracker(),
       true);
   fragments.emplace_back(std::move(fragment));
 
@@ -919,7 +943,8 @@ TEST_CASE_METHOD(
   uint64_t data2_size = data.size() * sizeof(int);
   write_1d_fragment(coords2.data(), &coords2_size, data2.data(), &data2_size);
 
-  total_budget_ = "1000000";
+  total_budget_ = "1900000";
+  tile_upper_memory_limit_ = "100000";
   ratio_array_data_ = set_subarray ? "0.003" : "0.002";
   partial_tile_offsets_loading_ = "true";
   update_config();
@@ -1007,8 +1032,8 @@ TEST_CASE_METHOD(
   }
 
   // Two result tile (2 * ~1208) will be bigger than the budget (1500).
-  total_budget_ = "10000";
-  ratio_coords_ = "0.15";
+  total_budget_ = "40000";
+  ratio_coords_ = "0.04";
   update_config();
 
   tiledb_array_t* array = nullptr;
@@ -1086,7 +1111,7 @@ TEST_CASE_METHOD(
   write_1d_fragment(coords, &coords_size, data, &data_size);
 
   // One result tile (~505) will be larger than leftover memory.
-  total_budget_ = "800";
+  total_budget_ = "1800";
   ratio_array_data_ = "0.99";
   ratio_coords_ = "0.0005";
   update_config();
@@ -1526,21 +1551,23 @@ TEST_CASE_METHOD(
   auto&& [array, fragments] = open_default_array_1d_with_fragments(capacity);
 
   // Make a vector of tiles.
-  std::vector<UnorderedWithDupsResultTile<uint64_t>> rt;
+  std::list<UnorderedWithDupsResultTile<uint64_t>> rt;
   for (uint64_t t = 0; t < num_tiles; t++) {
-    rt.emplace_back(0, t, *fragments[0]);
+    rt.emplace_back(
+        0, t, *fragments[0], tiledb::test::get_test_memory_tracker());
 
     // Allocate and set the bitmap if required.
     if (bitmaps[t].size() > 0) {
-      rt.back().bitmap() = bitmaps[t];
+      rt.back().bitmap().assign(bitmaps[t].begin(), bitmaps[t].end());
       rt.back().count_cells();
     }
   }
 
   // Create the result_tiles pointer vector.
   std::vector<ResultTile*> result_tiles(rt.size());
-  for (uint64_t i = 0; i < rt.size(); i++) {
-    result_tiles[i] = &rt[i];
+  uint64_t i = 0;
+  for (auto& t : rt) {
+    result_tiles[i++] = &t;
   }
 
   // Create a Query buffer.
@@ -1742,21 +1769,23 @@ TEST_CASE_METHOD(
   auto&& [array, fragments] = open_default_array_1d_with_fragments(capacity);
 
   // Make a vector of tiles.
-  std::vector<UnorderedWithDupsResultTile<uint64_t>> rt;
+  std::list<UnorderedWithDupsResultTile<uint64_t>> rt;
   for (uint64_t t = 0; t < num_tiles; t++) {
-    rt.emplace_back(0, t, *fragments[0]);
+    rt.emplace_back(
+        0, t, *fragments[0], tiledb::test::get_test_memory_tracker());
 
     // Allocate and set the bitmap if required.
     if (bitmaps[t].size() > 0) {
-      rt.back().bitmap() = bitmaps[t];
+      rt.back().bitmap().assign(bitmaps[t].begin(), bitmaps[t].end());
       rt.back().count_cells();
     }
   }
 
   // Create the result_tiles pointer vector.
   std::vector<ResultTile*> result_tiles(rt.size());
-  for (uint64_t i = 0; i < rt.size(); i++) {
-    result_tiles[i] = &rt[i];
+  uint64_t i = 0;
+  for (auto& t : rt) {
+    result_tiles[i++] = &t;
   }
 
   // Call the function.
