@@ -552,6 +552,26 @@ TEMPLATE_LIST_TEST_CASE("VFS: File I/O", "[vfs][uri][file_io]", AllBackends) {
   }
 }
 
+TEST_CASE("VFS: Test end-to-end", "[.vfs-e2e]") {
+  auto test_file_ptr = getenv("TILEDB_VFS_E2E_TEST_FILE");
+  if (test_file_ptr == nullptr) {
+    FAIL("TILEDB_VFS_E2E_TEST_FILE variable is not specified");
+  }
+  URI test_file{test_file_ptr};
+
+  ThreadPool compute_tp(1);
+  ThreadPool io_tp(1);
+  // Will be configured from environment variables.
+  Config config;
+
+  VFS vfs{&g_helper_stats, &compute_tp, &io_tp, config};
+  REQUIRE(vfs.supports_uri_scheme(test_file));
+
+  uint64_t nbytes = 0;
+  require_tiledb_ok(vfs.file_size(test_file, &nbytes));
+  CHECK(nbytes > 0);
+}
+
 TEST_CASE("VFS: test ls_with_sizes", "[vfs][ls-with-sizes]") {
   ThreadPool compute_tp(4);
   ThreadPool io_tp(4);
@@ -602,8 +622,9 @@ TEST_CASE("VFS: test ls_with_sizes", "[vfs][ls-with-sizes]") {
   require_tiledb_ok(vfs_ls.remove_dir(URI(path)));
 }
 
-// Currently only S3 is supported for VFS::ls_recursive.
-using TestBackends = std::tuple<S3Test>;
+// Currently only local, S3, Azure and GCS are supported for VFS::ls_recursive.
+// TODO: LocalFsTest currently fails. Fix and re-enable.
+using TestBackends = std::tuple</*LocalFsTest,*/ S3Test, AzureTest, GCSTest>;
 TEMPLATE_LIST_TEST_CASE(
     "VFS: Test internal ls_filtered recursion argument",
     "[vfs][ls_filtered][recursion]",
@@ -617,10 +638,9 @@ TEMPLATE_LIST_TEST_CASE(
   DYNAMIC_SECTION(
       fs.temp_dir_.backend_name()
       << " ls_filtered with recursion: " << (recursive ? "true" : "false")) {
-#ifdef HAVE_S3
     // If testing with recursion use the root directory, otherwise use a subdir.
     auto path = recursive ? fs.temp_dir_ : fs.temp_dir_.join_path("subdir_1");
-    auto ls_objects = fs.get_s3().ls_filtered(
+    auto ls_objects = fs.vfs_.ls_filtered(
         path, VFSTestBase::accept_all_files, accept_all_dirs, recursive);
 
     auto expected = fs.expected_results();
@@ -634,7 +654,6 @@ TEMPLATE_LIST_TEST_CASE(
 
     CHECK(ls_objects.size() == expected.size());
     CHECK(ls_objects == expected);
-#endif
   }
 }
 
@@ -649,12 +668,7 @@ TEST_CASE(
   }
   std::string backend = vfs_test.temp_dir_.backend_name();
 
-  if (vfs_test.temp_dir_.is_s3()) {
-    DYNAMIC_SECTION(backend << " supported backend should not throw") {
-      CHECK_NOTHROW(vfs_test.vfs_.ls_recursive(
-          vfs_test.temp_dir_, VFSTestBase::accept_all_files));
-    }
-  } else {
+  if (vfs_test.temp_dir_.is_hdfs()) {
     DYNAMIC_SECTION(backend << " unsupported backend should throw") {
       CHECK_THROWS_WITH(
           vfs_test.vfs_.ls_recursive(
@@ -662,13 +676,18 @@ TEST_CASE(
           Catch::Matchers::ContainsSubstring(
               "storage backend is not supported"));
     }
+  } else {
+    DYNAMIC_SECTION(backend << " supported backend should not throw") {
+      CHECK_NOTHROW(vfs_test.vfs_.ls_recursive(
+          vfs_test.temp_dir_, VFSTestBase::accept_all_files));
+    }
   }
 }
 
 TEST_CASE(
     "VFS: Throwing FileFilter for ls_recursive",
     "[vfs][ls_recursive][file-filter]") {
-  std::string prefix = "s3://";
+  std::string prefix = GENERATE("s3://", "azure://", "gcs://", "gs://");
   VFSTest vfs_test({0}, prefix);
   if (!vfs_test.is_supported()) {
     return;
