@@ -61,12 +61,20 @@ using namespace tiledb::common;
 
 namespace tiledb::sm {
 
+class WhiteboxConfig;
+
 /**
  * This class manages the TileDB configuration options.
  * It is implemented as a simple map from string to string.
  * Parsing to appropriate types happens on demand.
  */
 class Config {
+  friend class ConfigIter;
+  /**
+   * WhiteboxConfig makes available internals of Config for testing.
+   */
+  friend class WhiteboxConfig;
+
  public:
   /* ****************************** */
   /*        CONFIG DEFAULTS         */
@@ -219,6 +227,15 @@ class Config {
   /** Maximum memory budget for readers and writers. */
   static const std::string SM_MEM_TOTAL_BUDGET;
 
+  /** Weight for consolidation buffers used to split total budget. */
+  static const std::string SM_MEM_CONSOLIDATION_BUFFERS_WEIGHT;
+
+  /** Weight for consolidation read query used to split total budget. */
+  static const std::string SM_MEM_CONSOLIDATION_READER_WEIGHT;
+
+  /** Weight for consolidation write query used to split total budget. */
+  static const std::string SM_MEM_CONSOLIDATION_WRITER_WEIGHT;
+
   /** Ratio of the sparse global order reader budget used for coords. */
   static const std::string SM_MEM_SPARSE_GLOBAL_ORDER_RATIO_COORDS;
 
@@ -269,9 +286,6 @@ class Config {
   /** The buffer size for each attribute used in consolidation. */
   static const std::string SM_CONSOLIDATION_BUFFER_SIZE;
 
-  /** The total buffer size for all attributes during consolidation. */
-  static const std::string SM_CONSOLIDATION_TOTAL_BUFFER_SIZE;
-
   /** The maximum fragment size used in consolidation. */
   static const std::string SM_CONSOLIDATION_MAX_FRAGMENT_SIZE;
 
@@ -698,9 +712,6 @@ class Config {
   Status get_vector(
       const std::string& param, std::vector<T>* value, bool* found) const;
 
-  /** Returns the param -> value map. */
-  const std::map<std::string, std::string>& param_values() const;
-
   /** Gets the set parameters. */
   const std::set<std::string>& set_params() const;
 
@@ -715,6 +726,10 @@ class Config {
   /** Compares configs for equality. */
   bool operator==(const Config& rhs) const;
 
+  /** Get all config params taking into account environment variables */
+  const std::map<std::string, std::string> get_all_params_from_config_or_env()
+      const;
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
@@ -791,6 +806,9 @@ class Config {
 
   template <bool must_find_>
   optional<std::string> get_internal_string(const std::string& key) const;
+
+  /** Returns the param -> value map. */
+  const std::map<std::string, std::string>& param_values() const;
 };
 
 /**
