@@ -112,9 +112,8 @@ Status ArrayMetaConsolidator::consolidate(
   throw_if_not_ok(array_for_writes.close());
 
   // Write vacuum file
-  RETURN_NOT_OK(
-      storage_manager_->vfs()->write(vac_uri, data.c_str(), data.size()));
-  RETURN_NOT_OK(storage_manager_->vfs()->close_file(vac_uri));
+  throw_if_not_ok(resources_.vfs().write(vac_uri, data.c_str(), data.size()));
+  throw_if_not_ok(resources_.vfs().close_file(vac_uri));
 
   return Status::Ok();
 }
@@ -126,18 +125,14 @@ void ArrayMetaConsolidator::vacuum(const char* array_name) {
   }
 
   // Get the array metadata URIs and vacuum file URIs to be vacuum
-  auto vfs = storage_manager_->vfs();
-  auto compute_tp = storage_manager_->compute_tp();
-
+  auto& vfs = resources_.vfs();
+  auto& compute_tp = resources_.compute_tp();
   auto array_dir = ArrayDirectory(
-      storage_manager_->resources(),
-      URI(array_name),
-      0,
-      std::numeric_limits<uint64_t>::max());
+      resources_, URI(array_name), 0, std::numeric_limits<uint64_t>::max());
 
   // Delete the array metadata and vacuum files
-  vfs->remove_files(compute_tp, array_dir.array_meta_uris_to_vacuum());
-  vfs->remove_files(compute_tp, array_dir.array_meta_vac_uris_to_vacuum());
+  vfs.remove_files(&compute_tp, array_dir.array_meta_uris_to_vacuum());
+  vfs.remove_files(&compute_tp, array_dir.array_meta_vac_uris_to_vacuum());
 }
 
 /* ****************************** */
@@ -146,7 +141,7 @@ void ArrayMetaConsolidator::vacuum(const char* array_name) {
 
 Status ArrayMetaConsolidator::set_config(const Config& config) {
   // Set the consolidation config for ease of use
-  Config merged_config = storage_manager_->config();
+  Config merged_config = resources_.config();
   merged_config.inherit(config);
   bool found = false;
   RETURN_NOT_OK(merged_config.get<uint64_t>(
