@@ -70,7 +70,8 @@ void FragmentConsolidationWorkspace::resize_buffers(
     stats::Stats* stats,
     const FragmentConsolidationConfig& config,
     const ArraySchema& array_schema,
-    std::unordered_map<std::string, uint64_t>& avg_cell_sizes) {
+    std::unordered_map<std::string, uint64_t>& avg_cell_sizes,
+    uint64_t total_buffers_budget) {
   auto timer_se = stats->start_timer("resize_buffers");
 
   // For easy reference
@@ -133,15 +134,12 @@ void FragmentConsolidationWorkspace::resize_buffers(
     buffer_weights.emplace_back(sizeof(uint64_t));
   }
 
-  // Use the old buffer size setting to see how much memory we would use.
-  auto buffer_num = buffer_weights.size();
-  uint64_t total_budget = config.total_buffer_size_;
-
   // If a user set the per-attribute buffer size configuration, we override
   // the use of the total_budget_size config setting for backwards
   // compatible behavior.
+  auto buffer_num = buffer_weights.size();
   if (config.buffer_size_ != 0) {
-    total_budget = config.buffer_size_ * buffer_num;
+    total_buffers_budget = config.buffer_size_ * buffer_num;
   }
 
   // Calculate the size of individual buffers by assigning a weight based
@@ -149,7 +147,8 @@ void FragmentConsolidationWorkspace::resize_buffers(
   auto total_weights = std::accumulate(
       buffer_weights.begin(), buffer_weights.end(), static_cast<size_t>(0));
 
-  uint64_t adjusted_budget = total_budget / total_weights * total_weights;
+  uint64_t adjusted_budget =
+      total_buffers_budget / total_weights * total_weights;
 
   if (buffer_num > buffers_.size()) {
     sizes_.resize(buffer_num);
@@ -446,27 +445,31 @@ void FragmentConsolidator::vacuum(const char* array_name) {
     array_dir.write_commit_ignore_file(commit_uris_to_ignore);
   }
 
-  // Delete the vacuum files.
-  auto vfs = storage_manager_->vfs();
-  auto compute_tp = storage_manager_->compute_tp();
-  vfs->remove_files(
-      compute_tp, filtered_fragment_uris.fragment_vac_uris_to_vacuum());
-
   // Delete fragment directories
+  auto& vfs = resources_.vfs();
+  auto& compute_tp = resources_.compute_tp();
   throw_if_not_ok(parallel_for(
-      compute_tp, 0, fragment_uris_to_vacuum.size(), [&](size_t i) {
-        RETURN_NOT_OK(vfs->remove_dir(fragment_uris_to_vacuum[i]));
-
+      &compute_tp, 0, fragment_uris_to_vacuum.size(), [&](size_t i) {
         // Remove the commit file, if present.
         auto commit_uri = array_dir.get_commit_uri(fragment_uris_to_vacuum[i]);
         bool is_file = false;
-        RETURN_NOT_OK(vfs->is_file(commit_uri, &is_file));
+        throw_if_not_ok(vfs.is_file(commit_uri, &is_file));
         if (is_file) {
-          RETURN_NOT_OK(vfs->remove_file(commit_uri));
+          throw_if_not_ok(vfs.remove_file(commit_uri));
+        }
+
+        bool is_dir = false;
+        throw_if_not_ok(vfs.is_dir(fragment_uris_to_vacuum[i], &is_dir));
+        if (is_dir) {
+          throw_if_not_ok(vfs.remove_dir(fragment_uris_to_vacuum[i]));
         }
 
         return Status::Ok();
       }));
+
+  // Delete the vacuum files.
+  vfs.remove_files(
+      &compute_tp, filtered_fragment_uris.fragment_vac_uris_to_vacuum());
 }
 
 /* ****************************** */
@@ -548,25 +551,31 @@ Status FragmentConsolidator::consolidate_internal(
     }
   }
 
+  // Compute memory budgets
+  uint64_t total_weights =
+      config_.buffers_weight_ + config_.reader_weight_ + config_.writer_weight_;
+  uint64_t single_unit_budget = config_.total_budget_ / total_weights;
+  uint64_t buffers_budget = config_.buffers_weight_ * single_unit_budget;
+  uint64_t reader_budget = config_.reader_weight_ * single_unit_budget;
+  uint64_t writer_budget = config_.writer_weight_ * single_unit_budget;
+
   // Prepare buffers
   auto average_var_cell_sizes = array_for_reads->get_average_var_cell_sizes();
-  cw.resize_buffers(stats_, config_, array_schema, average_var_cell_sizes);
+  cw.resize_buffers(
+      stats_, config_, array_schema, average_var_cell_sizes, buffers_budget);
 
   // Create queries
-  auto query_r = (Query*)nullptr;
-  auto query_w = (Query*)nullptr;
-  auto st = create_queries(
+  tdb_unique_ptr<Query> query_r = nullptr;
+  tdb_unique_ptr<Query> query_w = nullptr;
+  throw_if_not_ok(create_queries(
       array_for_reads,
       array_for_writes,
       union_non_empty_domains,
-      &query_r,
-      &query_w,
-      new_fragment_uri);
-  if (!st.ok()) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
-    return st;
-  }
+      query_r,
+      query_w,
+      new_fragment_uri,
+      reader_budget,
+      writer_budget));
 
   // Get the vacuum URI
   URI vac_uri;
@@ -574,31 +583,20 @@ Status FragmentConsolidator::consolidate_internal(
     vac_uri =
         array_for_reads->array_directory().get_vacuum_uri(*new_fragment_uri);
   } catch (std::exception& e) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
     std::throw_with_nested(
         std::logic_error("[FragmentConsolidator::consolidate_internal] "));
   }
 
   // Read from one array and write to the other
-  try {
-    copy_array(query_r, query_w, cw);
-  } catch (...) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
-    throw;
-  };
+  copy_array(query_r.get(), query_w.get(), cw);
 
   // Finalize write query
-  st = query_w->finalize();
+  auto st = query_w->finalize();
   if (!st.ok()) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
     bool is_dir = false;
-    auto st2 = storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir);
-    (void)st2;  // Perhaps report this once we support an error stack
+    throw_if_not_ok(resources_.vfs().is_dir(*new_fragment_uri, &is_dir));
     if (is_dir)
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(*new_fragment_uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(*new_fragment_uri));
     return st;
   }
 
@@ -609,20 +607,13 @@ Status FragmentConsolidator::consolidate_internal(
       vac_uri,
       to_consolidate);
   if (!st.ok()) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
     bool is_dir = false;
-    throw_if_not_ok(
-        storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir));
+    throw_if_not_ok(resources_.vfs().is_dir(*new_fragment_uri, &is_dir));
     if (is_dir)
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(*new_fragment_uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(*new_fragment_uri));
     return st;
   }
 
-  // Clean up
-  tdb_delete(query_r);
-  tdb_delete(query_w);
-
   return st;
 }
 
@@ -662,9 +653,11 @@ Status FragmentConsolidator::create_queries(
     shared_ptr<Array> array_for_reads,
     shared_ptr<Array> array_for_writes,
     const NDRange& subarray,
-    Query** query_r,
-    Query** query_w,
-    URI* new_fragment_uri) {
+    tdb_unique_ptr<Query>& query_r,
+    tdb_unique_ptr<Query>& query_w,
+    URI* new_fragment_uri,
+    uint64_t read_memory_budget,
+    uint64_t write_memory_budget) {
   auto timer_se = stats_->start_timer("consolidate_create_queries");
 
   const auto dense = array_for_reads->array_schema_latest().dense();
@@ -674,48 +667,54 @@ Status FragmentConsolidator::create_queries(
   // is not a user input prone to errors).
 
   // Create read query
-  *query_r = tdb_new(Query, storage_manager_, array_for_reads);
-  RETURN_NOT_OK((*query_r)->set_layout(Layout::GLOBAL_ORDER));
+  query_r = tdb_unique_ptr<Query>(tdb_new(
+      Query, storage_manager_, array_for_reads, nullopt, read_memory_budget));
+  RETURN_NOT_OK(query_r->set_layout(Layout::GLOBAL_ORDER));
 
   // Dense consolidation will do a tile aligned read.
   if (dense) {
     NDRange read_subarray = subarray;
     auto& domain{array_for_reads->array_schema_latest().domain()};
     domain.expand_to_tiles(&read_subarray);
-    RETURN_NOT_OK((*query_r)->set_subarray_unsafe(read_subarray));
+    RETURN_NOT_OK(query_r->set_subarray_unsafe(read_subarray));
   }
 
   // Enable consolidation with timestamps on the reader, if applicable.
   if (config_.with_timestamps_ && !dense) {
-    throw_if_not_ok((*query_r)->set_consolidation_with_timestamps());
+    throw_if_not_ok(query_r->set_consolidation_with_timestamps());
   }
 
   // Get last fragment URI, which will be the URI of the consolidated fragment
-  auto first = (*query_r)->first_fragment_uri();
-  auto last = (*query_r)->last_fragment_uri();
+  auto first = query_r->first_fragment_uri();
+  auto last = query_r->last_fragment_uri();
 
   auto write_version = array_for_reads->array_schema_latest().write_version();
   auto fragment_name = storage_format::generate_consolidated_fragment_name(
       first, last, write_version);
 
   // Create write query
-  *query_w = tdb_new(Query, storage_manager_, array_for_writes, fragment_name);
-  RETURN_NOT_OK((*query_w)->set_layout(Layout::GLOBAL_ORDER));
-  RETURN_NOT_OK((*query_w)->disable_checks_consolidation());
-  (*query_w)->set_fragment_size(config_.max_fragment_size_);
+  query_w = tdb_unique_ptr<Query>(tdb_new(
+      Query,
+      storage_manager_,
+      array_for_writes,
+      fragment_name,
+      write_memory_budget));
+  RETURN_NOT_OK(query_w->set_layout(Layout::GLOBAL_ORDER));
+  RETURN_NOT_OK(query_w->disable_checks_consolidation());
+  query_w->set_fragment_size(config_.max_fragment_size_);
   if (array_for_reads->array_schema_latest().dense()) {
-    RETURN_NOT_OK((*query_w)->set_subarray_unsafe(subarray));
+    RETURN_NOT_OK(query_w->set_subarray_unsafe(subarray));
   }
 
   // Set the processed conditions on new fragment.
   const auto& delete_and_update_tiles_location =
-      (*query_r)->array()->array_directory().delete_and_update_tiles_location();
+      query_r->array()->array_directory().delete_and_update_tiles_location();
   std::vector<std::string> processed_conditions;
   processed_conditions.reserve(delete_and_update_tiles_location.size());
   for (auto& location : delete_and_update_tiles_location) {
     processed_conditions.emplace_back(location.condition_marker());
   }
-  (*query_w)->set_processed_conditions(processed_conditions);
+  query_w->set_processed_conditions(processed_conditions);
 
   // Set the URI for the new fragment.
   auto frag_uri =
@@ -928,7 +927,7 @@ void FragmentConsolidator::set_query_buffers(
 
 Status FragmentConsolidator::set_config(const Config& config) {
   // Set the consolidation config for ease of use
-  Config merged_config = storage_manager_->config();
+  Config merged_config = resources_.config();
   merged_config.inherit(config);
   bool found = false;
   config_.amplification_ = 0.0f;
@@ -941,21 +940,31 @@ Status FragmentConsolidator::set_config(const Config& config) {
   assert(found);
   config_.buffer_size_ = 0;
   // Only set the buffer_size_ if the user specified a value. Otherwise, we use
-  // the new sm.consolidation.total_buffer_size instead.
+  // the new sm.mem.consolidation.buffers_weight instead.
   if (merged_config.set_params().count("sm.consolidation.buffer_size") > 0) {
     logger_->warn(
         "The `sm.consolidation.buffer_size configuration setting has been "
         "deprecated. Set consolidation buffer sizes using the newer "
-        "`sm.consolidation.total_buffer_size` setting.");
+        "`sm.mem.consolidation.buffers_weight` setting.");
     RETURN_NOT_OK(merged_config.get<uint64_t>(
         "sm.consolidation.buffer_size", &config_.buffer_size_, &found));
     assert(found);
   }
-  config_.total_buffer_size_ = 0;
+  config_.total_budget_ = 0;
   RETURN_NOT_OK(merged_config.get<uint64_t>(
-      "sm.consolidation.total_buffer_size",
-      &config_.total_buffer_size_,
-      &found));
+      "sm.mem.total_budget", &config_.total_budget_, &found));
+  assert(found);
+  config_.buffers_weight_ = 0;
+  RETURN_NOT_OK(merged_config.get<uint64_t>(
+      "sm.mem.consolidation.buffers_weight", &config_.buffers_weight_, &found));
+  assert(found);
+  config_.reader_weight_ = 0;
+  RETURN_NOT_OK(merged_config.get<uint64_t>(
+      "sm.mem.consolidation.reader_weight", &config_.reader_weight_, &found));
+  assert(found);
+  config_.writer_weight_ = 0;
+  RETURN_NOT_OK(merged_config.get<uint64_t>(
+      "sm.mem.consolidation.writer_weight", &config_.writer_weight_, &found));
   assert(found);
   config_.max_fragment_size_ = 0;
   RETURN_NOT_OK(merged_config.get<uint64_t>(
@@ -1030,9 +1039,8 @@ Status FragmentConsolidator::write_vacuum_file(
   }
 
   auto data = ss.str();
-  RETURN_NOT_OK(
-      storage_manager_->vfs()->write(vac_uri, data.c_str(), data.size()));
-  RETURN_NOT_OK(storage_manager_->vfs()->close_file(vac_uri));
+  throw_if_not_ok(resources_.vfs().write(vac_uri, data.c_str(), data.size()));
+  throw_if_not_ok(resources_.vfs().close_file(vac_uri));
 
   return Status::Ok();
 }
