@@ -239,12 +239,12 @@ class S3_within_VFS {
   }
 
   /** Protected accessor for the S3 object. */
-  inline tiledb::sm::S3& s3() {
+  inline S3& s3() {
     return s3_;
   }
 
   /** Protected accessor for the const S3 object. */
-  inline const tiledb::sm::S3& s3() const {
+  inline const S3& s3() const {
     return s3_;
   }
 };
@@ -332,11 +332,11 @@ class VFS : private VFSBase, protected S3_within_VFS {
    * @param path The input path.
    * @return The string with the absolute path.
    */
-  static std::string abs_path(const std::string& path);
+  static std::string abs_path(std::string_view path);
 
   /**
    * Return a config object containing the VFS parameters. All other non-VFS
-   * parameters will are set to default values.
+   * parameters are set to default values.
    */
   Config config() const;
 
@@ -514,27 +514,56 @@ class VFS : private VFSBase, protected S3_within_VFS {
    * the FilePredicate on each entry collected and the DirectoryPredicate on
    * common prefixes for pruning.
    *
-   * Currently only S3 is supported for ls_recursive.
+   * Currently this API is only supported for local files, S3 and Azure.
    *
    * @param parent The parent prefix to list sub-paths.
    * @param f The FilePredicate to invoke on each object for filtering.
    * @param d The DirectoryPredicate to invoke on each common prefix for
    *    pruning. This is currently unused, but is kept here for future support.
+   * @param recursive Whether to list the objects recursively.
    * @return Vector of results with each entry being a pair of the string URI
    *    and object size.
    */
   template <FilePredicate F, DirectoryPredicate D = DirectoryFilter>
-  LsObjects ls_recursive(
+  LsObjects ls_filtered(
       const URI& parent,
       [[maybe_unused]] F f,
-      [[maybe_unused]] D d = accept_all_dirs) const {
+      [[maybe_unused]] D d,
+      bool recursive) const {
     LsObjects results;
     try {
-      if (parent.is_s3()) {
+      if (parent.is_file()) {
+#ifdef _WIN32
+        results = win_.ls_filtered(parent, f, d, recursive);
+#else
+        results = posix_.ls_filtered(parent, f, d, recursive);
+#endif
+      } else if (parent.is_s3()) {
 #ifdef HAVE_S3
-        results = s3().ls_filtered(parent, f, d, true);
+        results = s3().ls_filtered(parent, f, d, recursive);
 #else
         throw filesystem::VFSException("TileDB was built without S3 support");
+#endif
+      } else if (parent.is_gcs()) {
+#ifdef HAVE_GCS
+        results = gcs_.ls_filtered(parent, f, d, recursive);
+#else
+        throw filesystem::VFSException("TileDB was built without GCS support");
+#endif
+      } else if (parent.is_azure()) {
+#ifdef HAVE_AZURE
+        results = azure_.ls_filtered(parent, f, d, recursive);
+#else
+        throw filesystem::VFSException(
+            "TileDB was built without Azure support");
+#endif
+      } else if (parent.is_hdfs()) {
+#ifdef HAVE_HDFS
+        throw filesystem::VFSException(
+            "Recursive ls over " + parent.backend_name() +
+            " storage backend is not supported.");
+#else
+        throw filesystem::VFSException("TileDB was built without HDFS support");
 #endif
       } else {
         throw filesystem::VFSException(
@@ -550,6 +579,28 @@ class VFS : private VFSBase, protected S3_within_VFS {
     return results;
   }
 
+  /**
+   * Recursively lists objects and object information that start with `prefix`,
+   * invoking the FilePredicate on each entry collected and the
+   * DirectoryPredicate on common prefixes for pruning.
+   *
+   * Currently this API is only supported for local files, S3, Azure and GCS.
+   *
+   * @param parent The parent prefix to list sub-paths.
+   * @param f The FilePredicate to invoke on each object for filtering.
+   * @param d The DirectoryPredicate to invoke on each common prefix for
+   *    pruning. This is currently unused, but is kept here for future support.
+   * @return Vector of results with each entry being a pair of the string URI
+   *    and object size.
+   */
+  template <FilePredicate F, DirectoryPredicate D = DirectoryFilter>
+  LsObjects ls_recursive(
+      const URI& parent,
+      [[maybe_unused]] F f,
+      [[maybe_unused]] D d = accept_all_dirs) const {
+    return ls_filtered(parent, f, d, true);
+  }
+
   /**
    * Renames a file.
    *
@@ -850,8 +901,9 @@ class VFS : private VFSBase, protected S3_within_VFS {
 
       const uint64_t size = buffer.size();
       ReadAheadBuffer ra_buffer(offset, std::move(buffer));
-      return LRUCache<std::string, ReadAheadBuffer>::insert(
+      LRUCache<std::string, ReadAheadBuffer>::insert(
           uri.to_string(), std::move(ra_buffer), size);
+      return Status::Ok();
     }
 
    private:
