@@ -35,6 +35,7 @@
 #include "tiledb/common/logger.h"
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/stdx_string.h"
+#include "tiledb/sm/array/array.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/encryption_type.h"
 #include "tiledb/sm/enums/query_type.h"
@@ -55,9 +56,9 @@ using namespace tiledb::common;
 
 namespace tiledb::sm {
 
-class GroupStatusException : public StatusException {
+class GroupException : public StatusException {
  public:
-  explicit GroupStatusException(const std::string& message)
+  explicit GroupException(const std::string& message)
       : StatusException("Group", message) {
   }
 };
@@ -69,7 +70,7 @@ Group::Group(
     : memory_tracker_(resources.create_memory_tracker())
     , group_uri_(group_uri)
     , storage_manager_(storage_manager)
-    , config_(storage_manager_->config())
+    , config_(resources.config())
     , remote_(group_uri.is_tiledb())
     , metadata_(memory_tracker_)
     , metadata_loaded_(false)
@@ -82,16 +83,16 @@ Group::Group(
   memory_tracker_->set_type(MemoryTrackerType::GROUP);
 }
 
-Status Group::open(
+void Group::open(
     QueryType query_type, uint64_t timestamp_start, uint64_t timestamp_end) {
   // Checks
   if (is_open_) {
-    return Status_GroupError("Cannot open group; Group already open");
+    throw GroupException("Cannot open group; Group already open");
   }
 
   if (query_type != QueryType::READ && query_type != QueryType::WRITE &&
       query_type != QueryType::MODIFY_EXCLUSIVE) {
-    return Status_GroupError("Cannot open group; Unsupported query type");
+    throw GroupException("Cannot open group; Unsupported query type");
   }
 
   if (timestamp_end == UINT64_MAX) {
@@ -125,7 +126,7 @@ Status Group::open(
     encryption_type_from_cfg = config_.get("sm.encryption_type", &found);
     assert(found);
     auto [st, et] = encryption_type_enum(encryption_type_from_cfg);
-    RETURN_NOT_OK(st);
+    throw_if_not_ok(st);
     encryption_type = et.value();
 
     if (!EncryptionKey::is_valid_key_length(
@@ -136,58 +137,47 @@ Status Group::open(
     }
   }
 
-  if (remote_ && encryption_type != EncryptionType::NO_ENCRYPTION)
-    return Status_GroupError(
+  if (remote_ && encryption_type != EncryptionType::NO_ENCRYPTION) {
+    throw GroupException(
         "Cannot open group; encrypted remote groups are not supported.");
+  }
 
   // Copy the key bytes.
-  RETURN_NOT_OK(
+  throw_if_not_ok(
       encryption_key_->set_key(encryption_type, encryption_key, key_length));
 
   metadata_.clear();
   metadata_loaded_ = false;
 
   if (remote_) {
-    auto rest_client = storage_manager_->rest_client();
+    auto rest_client = resources_.rest_client();
     if (rest_client == nullptr) {
-      return Status_GroupError(
+      throw GroupException(
           "Cannot open group; remote group with no REST client.");
     }
 
     // Set initial group details to be deserialized into
     group_details_ = tdb::make_shared<GroupDetailsV2>(HERE(), group_uri_);
 
-    RETURN_NOT_OK(rest_client->post_group_from_rest(group_uri_, this));
+    throw_if_not_ok(rest_client->post_group_from_rest(group_uri_, this));
   } else if (query_type == QueryType::READ) {
-    try {
-      group_dir_ = make_shared<GroupDirectory>(
-          HERE(),
-          storage_manager_->vfs(),
-          storage_manager_->compute_tp(),
-          group_uri_,
-          timestamp_start,
-          timestamp_end);
-    } catch (const std::logic_error& le) {
-      return Status_GroupDirectoryError(le.what());
-    }
-
+    group_dir_ = make_shared<GroupDirectory>(
+        HERE(),
+        resources_.vfs(),
+        resources_.compute_tp(),
+        group_uri_,
+        timestamp_start,
+        timestamp_end);
     group_open_for_reads();
   } else {
-    try {
-      group_dir_ = make_shared<GroupDirectory>(
-          HERE(),
-          storage_manager_->vfs(),
-          storage_manager_->compute_tp(),
-          group_uri_,
-          timestamp_start,
-          (timestamp_end != 0) ? timestamp_end :
-                                 utils::time::timestamp_now_ms());
-    } catch (const std::logic_error& le) {
-      return Status_GroupDirectoryError(le.what());
-    }
-
+    group_dir_ = make_shared<GroupDirectory>(
+        HERE(),
+        resources_.vfs(),
+        resources_.compute_tp(),
+        group_uri_,
+        timestamp_start,
+        (timestamp_end != 0) ? timestamp_end : utils::time::timestamp_now_ms());
     group_open_for_writes();
-
     metadata_.reset(timestamp_end);
   }
 
@@ -198,26 +188,37 @@ Status Group::open(
 
   query_type_ = query_type;
   is_open_ = true;
-
-  return Status::Ok();
 }
 
-Status Group::open(QueryType query_type) {
-  bool found = false;
-  RETURN_NOT_OK(config_.get<uint64_t>(
-      "sm.group.timestamp_start", &timestamp_start_, &found));
-  assert(found);
-  RETURN_NOT_OK(
-      config_.get<uint64_t>("sm.group.timestamp_end", &timestamp_end_, &found));
-  assert(found);
+void Group::open(QueryType query_type) {
+  timestamp_start_ =
+      config_.get<uint64_t>("sm.group.timestamp_start", Config::must_find);
+  timestamp_end_ =
+      config_.get<uint64_t>("sm.group.timestamp_end", Config::must_find);
 
-  return Group::open(query_type, timestamp_start_, timestamp_end_);
+  Group::open(query_type, timestamp_start_, timestamp_end_);
 }
 
-Status Group::close() {
+void Group::close_for_writes() {
+  // Flush the group metadata
+  unsafe_metadata()->store(resources_, group_uri(), *encryption_key());
+
+  // Store any changes required
+  if (group_details()->is_modified()) {
+    const URI& group_detail_folder_uri = group_detail_uri();
+    auto group_detail_uri = generate_detail_uri();
+    throw_if_not_ok(group_details()->store(
+        resources_,
+        group_detail_folder_uri,
+        group_detail_uri,
+        *encryption_key()));
+  }
+}
+
+void Group::close() {
   // Check if group is open
   if (!is_open_)
-    return Status_GroupError("Cannot close group; Group not open.");
+    return;
 
   if (remote_) {
     // Update group metadata for write queries if metadata was written by the
@@ -228,34 +229,36 @@ Status Group::close() {
         // Set metadata loaded to be true so when serialization fetches the
         // metadata it won't trigger a deadlock
         metadata_loaded_ = true;
-        auto rest_client = storage_manager_->rest_client();
-        if (rest_client == nullptr)
-          return Status_GroupError(
+        auto rest_client = resources_.rest_client();
+        if (rest_client == nullptr) {
+          throw GroupException(
               "Error closing group; remote group with no REST client.");
-        RETURN_NOT_OK(
+        }
+        throw_if_not_ok(
             rest_client->put_group_metadata_to_rest(group_uri_, this));
       }
       if (!members_to_modify().empty()) {
-        auto rest_client = storage_manager_->rest_client();
-        if (rest_client == nullptr)
-          return Status_GroupError(
+        auto rest_client = resources_.rest_client();
+        if (rest_client == nullptr) {
+          throw GroupException(
               "Error closing group; remote group with no REST client.");
-        RETURN_NOT_OK(rest_client->patch_group_to_rest(group_uri_, this));
+        }
+        throw_if_not_ok(rest_client->patch_group_to_rest(group_uri_, this));
       }
     }
     // Storage manager does not own the group schema for remote groups.
   } else {
     if (query_type_ == QueryType::READ) {
-      RETURN_NOT_OK(storage_manager_->group_close_for_reads(this));
+      close_for_reads();
     } else if (
         query_type_ == QueryType::WRITE ||
         query_type_ == QueryType::MODIFY_EXCLUSIVE) {
       try {
-        throw_if_not_ok(storage_manager_->group_close_for_writes(this));
+        close_for_writes();
       } catch (StatusException& exc) {
-        std::string msg = exc.what();
-        msg += " : Was storage for the group moved or deleted before closing?";
-        throw GroupStatusException(msg);
+        throw GroupException(
+            std::string(exc.what()) +
+            " : Was storage for the group moved or deleted before closing?");
       }
     }
   }
@@ -264,7 +267,6 @@ Status Group::close() {
   metadata_loaded_ = false;
   is_open_ = false;
   clear();
-  return Status::Ok();
 }
 
 bool Group::is_open() const {
@@ -283,35 +285,32 @@ const shared_ptr<GroupDetails> Group::group_details() const {
   return group_details_;
 }
 
-Status Group::get_query_type(QueryType* query_type) const {
+QueryType Group::get_query_type() const {
   // Error if the group is not open
-  if (!is_open_)
-    return LOG_STATUS(
-        Status_GroupError("Cannot get query_type; Group is not open"));
+  if (!is_open_) {
+    throw GroupException("Cannot get query_type; Group is not open");
+  }
 
-  *query_type = query_type_;
-
-  return Status::Ok();
+  return query_type_;
 }
 
 void Group::delete_group(const URI& uri, bool recursive) {
   // Check that group is open
   if (!is_open_) {
-    throw GroupStatusException("[delete_group] Group is not open");
+    throw GroupException("[delete_group] Group is not open");
   }
 
   // Check that query type is MODIFY_EXCLUSIVE
   if (query_type_ != QueryType::MODIFY_EXCLUSIVE) {
-    throw GroupStatusException(
-        "[delete_group] Query type must be MODIFY_EXCLUSIVE");
+    throw GroupException("[delete_group] Query type must be MODIFY_EXCLUSIVE");
   }
 
   // Delete group data
   if (remote_) {
-    auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
-      throw GroupStatusException(
-          "[delete_group] Remote group with no REST client.");
+    auto rest_client = resources_.rest_client();
+    if (rest_client == nullptr) {
+      throw GroupException("[delete_group] Remote group with no REST client.");
+    }
     rest_client->delete_group_from_rest(uri, recursive);
   } else {
     // Delete group members within the group when deleting recursively
@@ -324,39 +323,62 @@ void Group::delete_group(const URI& uri, bool recursive) {
         }
 
         if (member->type() == ObjectType::ARRAY) {
-          storage_manager_->delete_array(member_uri.to_string().c_str());
+          Array::delete_array(resources_, member_uri);
         } else if (member->type() == ObjectType::GROUP) {
           Group group_rec(resources_, member_uri, storage_manager_);
-          throw_if_not_ok(group_rec.open(QueryType::MODIFY_EXCLUSIVE));
+          group_rec.open(QueryType::MODIFY_EXCLUSIVE);
           group_rec.delete_group(member_uri, true);
         }
       }
     }
-    storage_manager_->delete_group(uri.c_str());
+
+    auto& vfs = resources_.vfs();
+    auto& compute_tp = resources_.compute_tp();
+    auto group_dir = GroupDirectory(
+        vfs, compute_tp, uri, 0, std::numeric_limits<uint64_t>::max());
+
+    // Delete the group detail, group metadata and group files
+    vfs.remove_files(&compute_tp, group_dir.group_detail_uris());
+    vfs.remove_files(&compute_tp, group_dir.group_meta_uris());
+    vfs.remove_files(&compute_tp, group_dir.group_meta_uris_to_vacuum());
+    vfs.remove_files(&compute_tp, group_dir.group_meta_vac_uris_to_vacuum());
+    vfs.remove_files(&compute_tp, group_dir.group_file_uris());
+
+    // Delete all tiledb child directories
+    // Note: using vfs().ls() here could delete user data
+    std::vector<URI> dirs;
+    auto parent_dir = group_dir.uri().c_str();
+    for (auto group_dir_name : constants::group_dir_names) {
+      dirs.emplace_back(URI(parent_dir + group_dir_name));
+    }
+    vfs.remove_dirs(&compute_tp, dirs);
   }
   // Clear metadata and other pending changes to avoid patching a deleted group.
   metadata_.clear();
   group_details_->clear();
 
   // Close the deleted group
-  throw_if_not_ok(this->close());
+  this->close();
 }
 
 void Group::delete_metadata(const char* key) {
   // Check if group is open
-  if (!is_open_)
-    throw GroupStatusException("Cannot delete metadata. Group is not open");
+  if (!is_open_) {
+    throw GroupException("Cannot delete metadata. Group is not open");
+  }
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
-      query_type_ != QueryType::MODIFY_EXCLUSIVE)
-    throw GroupStatusException(
+      query_type_ != QueryType::MODIFY_EXCLUSIVE) {
+    throw GroupException(
         "Cannot delete metadata. Group was not opened in write or "
         "modify_exclusive mode");
+  }
 
   // Check if key is null
-  if (key == nullptr)
-    throw GroupStatusException("Cannot delete metadata. Key cannot be null");
+  if (key == nullptr) {
+    throw GroupException("Cannot delete metadata. Key cannot be null");
+  }
 
   metadata_.del(key);
 }
@@ -367,23 +389,27 @@ void Group::put_metadata(
     uint32_t value_num,
     const void* value) {
   // Check if group is open
-  if (!is_open_)
-    throw GroupStatusException("Cannot put metadata; Group is not open");
+  if (!is_open_) {
+    throw GroupException("Cannot put metadata; Group is not open");
+  }
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
-      query_type_ != QueryType::MODIFY_EXCLUSIVE)
-    throw GroupStatusException(
+      query_type_ != QueryType::MODIFY_EXCLUSIVE) {
+    throw GroupException(
         "Cannot put metadata; Group was not opened in write or "
         "modify_exclusive mode");
+  }
 
   // Check if key is null
-  if (key == nullptr)
-    throw GroupStatusException("Cannot put metadata; Key cannot be null");
+  if (key == nullptr) {
+    throw GroupException("Cannot put metadata; Key cannot be null");
+  }
 
   // Check if value type is ANY
-  if (value_type == Datatype::ANY)
-    throw GroupStatusException("Cannot put metadata; Value type cannot be ANY");
+  if (value_type == Datatype::ANY) {
+    throw GroupException("Cannot put metadata; Value type cannot be ANY");
+  }
 
   metadata_.put(key, value_type, value_num, value);
 }
@@ -394,21 +420,25 @@ void Group::get_metadata(
     uint32_t* value_num,
     const void** value) {
   // Check if group is open
-  if (!is_open_)
-    throw GroupStatusException("Cannot get metadata; Group is not open");
+  if (!is_open_) {
+    throw GroupException("Cannot get metadata; Group is not open");
+  }
 
   // Check mode
-  if (query_type_ != QueryType::READ)
-    throw GroupStatusException(
+  if (query_type_ != QueryType::READ) {
+    throw GroupException(
         "Cannot get metadata; Group was not opened in read mode");
+  }
 
   // Check if key is null
-  if (key == nullptr)
-    throw GroupStatusException("Cannot get metadata; Key cannot be null");
+  if (key == nullptr) {
+    throw GroupException("Cannot get metadata; Key cannot be null");
+  }
 
   // Load group metadata, if not loaded yet
-  if (!metadata_loaded_)
+  if (!metadata_loaded_) {
     load_metadata();
+  }
 
   metadata_.get(key, value_type, value_num, value);
 }
@@ -421,56 +451,65 @@ void Group::get_metadata(
     uint32_t* value_num,
     const void** value) {
   // Check if group is open
-  if (!is_open_)
-    throw GroupStatusException("Cannot get metadata; Group is not open");
+  if (!is_open_) {
+    throw GroupException("Cannot get metadata; Group is not open");
+  }
 
   // Check mode
-  if (query_type_ != QueryType::READ)
-    throw GroupStatusException(
+  if (query_type_ != QueryType::READ) {
+    throw GroupException(
         "Cannot get metadata; Group was not opened in read mode");
+  }
 
   // Load group metadata, if not loaded yet
-  if (!metadata_loaded_)
+  if (!metadata_loaded_) {
     load_metadata();
+  }
 
   metadata_.get(index, key, key_len, value_type, value_num, value);
 }
 
 uint64_t Group::get_metadata_num() {
   // Check if group is open
-  if (!is_open_)
-    throw GroupStatusException(
-        "Cannot get number of metadata; Group is not open");
+  if (!is_open_) {
+    throw GroupException("Cannot get number of metadata; Group is not open");
+  }
 
   // Check mode
-  if (query_type_ != QueryType::READ)
-    throw GroupStatusException(
+  if (query_type_ != QueryType::READ) {
+    throw GroupException(
         "Cannot get number of metadata; Group was not opened in read mode");
+  }
 
   // Load group metadata, if not loaded yet
-  if (!metadata_loaded_)
+  if (!metadata_loaded_) {
     load_metadata();
+  }
 
   return metadata_.num();
 }
 
 std::optional<Datatype> Group::metadata_type(const char* key) {
   // Check if group is open
-  if (!is_open_)
-    throw GroupStatusException("Cannot get metadata; Group is not open");
+  if (!is_open_) {
+    throw GroupException("Cannot get metadata; Group is not open");
+  }
 
   // Check mode
-  if (query_type_ != QueryType::READ)
-    throw GroupStatusException(
+  if (query_type_ != QueryType::READ) {
+    throw GroupException(
         "Cannot get metadata; Group was not opened in read mode");
+  }
 
   // Check if key is null
-  if (key == nullptr)
-    throw GroupStatusException("Cannot get metadata; Key cannot be null");
+  if (key == nullptr) {
+    throw GroupException("Cannot get metadata; Key cannot be null");
+  }
 
   // Load group metadata, if not loaded yet
-  if (!metadata_loaded_)
+  if (!metadata_loaded_) {
     load_metadata();
+  }
 
   return metadata_.metadata_type(key);
 }
@@ -505,7 +544,7 @@ const Config& Group::config() const {
 
 void Group::set_config(Config config) {
   if (is_open()) {
-    throw GroupStatusException("[set_config] Cannot set config; Group is open");
+    throw GroupException("[set_config] Cannot set config; Group is open");
   }
   config_.inherit(config);
 }
@@ -524,46 +563,43 @@ void Group::delete_member(const shared_ptr<GroupMember> group_member) {
   group_details_->delete_member(group_member);
 }
 
-Status Group::mark_member_for_addition(
+void Group::mark_member_for_addition(
     const URI& group_member_uri,
     const bool& relative,
     std::optional<std::string>& name) {
   std::lock_guard<std::mutex> lck(mtx_);
   // Check if group is open
   if (!is_open_) {
-    return Status_GroupError("Cannot add member; Group is not open");
+    throw GroupException("Cannot add member; Group is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
       query_type_ != QueryType::MODIFY_EXCLUSIVE) {
-    return Status_GroupError(
+    throw GroupException(
         "Cannot get member; Group was not opened in write or modify_exclusive "
         "mode");
   }
   group_details_->mark_member_for_addition(
-      group_member_uri, relative, name, storage_manager_);
-  return Status::Ok();
+      resources_, group_member_uri, relative, name);
 }
 
-Status Group::mark_member_for_removal(const std::string& name) {
+void Group::mark_member_for_removal(const std::string& name) {
   std::lock_guard<std::mutex> lck(mtx_);
   // Check if group is open
   if (!is_open_) {
-    return Status_GroupError(
-        "Cannot mark member for removal; Group is not open");
+    throw GroupException("Cannot mark member for removal; Group is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::WRITE &&
       query_type_ != QueryType::MODIFY_EXCLUSIVE) {
-    return Status_GroupError(
+    throw GroupException(
         "Cannot get member; Group was not opened in write or modify_exclusive "
         "mode");
   }
 
   group_details_->mark_member_for_removal(name);
-  return Status::Ok();
 }
 
 const std::vector<shared_ptr<GroupMember>>& Group::members_to_modify() const {
@@ -601,12 +637,12 @@ uint64_t Group::member_count() const {
   std::lock_guard<std::mutex> lck(mtx_);
   // Check if group is open
   if (!is_open_) {
-    throw Status_GroupError("Cannot get member count; Group is not open");
+    throw GroupException("Cannot get member count; Group is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    throw Status_GroupError(
+    throw GroupException(
         "Cannot get member; Group was not opened in read mode");
   }
 
@@ -619,12 +655,12 @@ tuple<std::string, ObjectType, optional<std::string>> Group::member_by_index(
 
   // Check if group is open
   if (!is_open_) {
-    throw Status_GroupError("Cannot get member by index; Group is not open");
+    throw GroupException("Cannot get member by index; Group is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    throw Status_GroupError(
+    throw GroupException(
         "Cannot get member; Group was not opened in read mode");
   }
 
@@ -637,12 +673,12 @@ Group::member_by_name(const std::string& name) {
 
   // Check if group is open
   if (!is_open_) {
-    throw Status_GroupError("Cannot get member by name; Group is not open");
+    throw GroupException("Cannot get member by name; Group is not open");
   }
 
   // Check mode
   if (query_type_ != QueryType::READ) {
-    throw Status_GroupError(
+    throw GroupException(
         "Cannot get member; Group was not opened in read mode");
   }
 
@@ -666,31 +702,26 @@ std::string Group::dump(
 
   for (const auto& member_entry : members()) {
     const auto& it = member_entry.second;
-    URI member_uri = it->uri();
-    if (it->relative()) {
-      member_uri = group_uri_.join_path(it->uri().to_string());
-    }
-
     ss << "|" << indent << l_indent << " " << *it;
-    bool do_recurse = false;
     if (it->type() == ObjectType::GROUP && recursive) {
-      // Before listing the group's members, check if the group exists in
-      // storage. If it does not, leave a message in the string.
-      ObjectType member_type = ObjectType::INVALID;
-      throw_if_not_ok(storage_manager_->object_type(member_uri, &member_type));
-      if (member_type == ObjectType::GROUP) {
-        do_recurse = true;
-      } else {
-        ss << " (does not exist)";
+      URI member_uri = it->uri();
+      if (it->relative()) {
+        member_uri = group_uri_.join_path(it->uri().to_string());
       }
-    }
-    ss << std::endl;
 
-    if (do_recurse) {
       Group group_rec(resources_, member_uri, storage_manager_);
-      throw_if_not_ok(group_rec.open(QueryType::READ));
-      ss << group_rec.dump(indent_size, num_indents + 2, recursive, false);
-      throw_if_not_ok(group_rec.close());
+      try {
+        group_rec.open(QueryType::READ);
+        ss << std::endl;
+        ss << group_rec.dump(indent_size, num_indents + 2, recursive, false);
+        group_rec.close();
+      } catch (GroupNotFoundException&) {
+        // If the group no longer exists in storage it will be listed but we
+        // won't be able to dump its members
+        ss << " (does not exist)" << std::endl;
+      }
+    } else {
+      ss << std::endl;
     }
   }
 
@@ -704,9 +735,10 @@ std::string Group::dump(
 void Group::load_metadata() {
   if (remote_) {
     auto rest_client = resources_.rest_client();
-    if (rest_client == nullptr)
-      throw GroupStatusException(
+    if (rest_client == nullptr) {
+      throw GroupException(
           "Cannot load metadata; remote group with no REST client.");
+    }
     throw_if_not_ok(
         rest_client->post_group_metadata_from_rest(group_uri_, this));
   } else {
@@ -737,7 +769,7 @@ void Group::load_metadata_from_storage(
             uri,
             0,
             encryption_key,
-            storage_manager_->resources().ephemeral_memory_tracker());
+            resources_.ephemeral_memory_tracker());
 
         return Status::Ok();
       }));
