@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2022 TileDB, Inc.
+ * @copyright Copyright (c) 2022-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,12 +41,12 @@
 #include "tiledb/sm/group/group_directory.h"
 #include "tiledb/sm/group/group_member.h"
 #include "tiledb/sm/metadata/metadata.h"
+#include "tiledb/sm/storage_manager/context_resources.h"
 #include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class GroupDetails {
  public:
@@ -63,15 +63,16 @@ class GroupDetails {
   /**
    * Add a member to a group, this will be flushed to disk on close
    *
+   * @param resources the context resources
    * @param group_member_uri group member uri
    * @param relative is this URI relative
    * @param name optional name for member
    */
   void mark_member_for_addition(
+      ContextResources& resources,
       const URI& group_member_uri,
       const bool& relative,
-      std::optional<std::string>& name,
-      StorageManager* storage_manager);
+      std::optional<std::string>& name);
 
   /**
    * Remove a member from a group, this will be flushed to disk on close
@@ -90,6 +91,25 @@ class GroupDetails {
    */
   const std::vector<shared_ptr<GroupMember>>& members_to_modify() const;
 
+  /**
+   * Get whether the group has been modified.
+   *
+   * This determines whether to write the group details on close.
+   */
+  bool is_modified() const {
+    return is_modified_;
+  }
+
+  /**
+   * Marks the group as modified.
+   *
+   * Used only by serialization, to support writing the group details of a
+   * deserialized group.
+   */
+  void set_modified() {
+    is_modified_ = true;
+  }
+
   /**
    * Get the unordered map of members
    * @return members
@@ -112,13 +132,31 @@ class GroupDetails {
    */
   void delete_member(const shared_ptr<GroupMember> group_member);
 
+  /**
+   * Store the group details
+   *
+   * @param resources the context resources
+   * @param group_detail_folder_uri group details folder
+   * @param group_detail_uri uri for detail file to write
+   * @param encryption_key encryption key for at-rest encryption
+   * @return status
+   */
+  Status store(
+      ContextResources& resources,
+      const URI& group_detail_folder_uri,
+      const URI& group_detail_uri,
+      const EncryptionKey& encryption_key);
+
   /**
    * Serializes the object members into a binary buffer.
    *
-   * @param buff The buffer to serialize the data into.
-   * @param version The format spec version.
+   * @param members The members to serialize. Should be retrieved from
+   * members_to_serialize().
+   * @param serializer The buffer to serialize the data into.
    */
-  virtual void serialize(Serializer& serializer);
+  virtual void serialize(
+      const std::vector<std::shared_ptr<GroupMember>>& members,
+      Serializer& serializer) const = 0;
 
   /**
    * Returns a Group object from the data in the input binary buffer.
@@ -145,12 +183,6 @@ class GroupDetails {
   /** Returns the group URI. */
   const URI& group_uri() const;
 
-  /**
-   * Have changes been applied to a group in write mode
-   * @return changes_applied_
-   */
-  bool changes_applied() const;
-
   /**
    * Get count of members
    *
@@ -184,11 +216,11 @@ class GroupDetails {
   format_version_t version() const;
 
   /**
-   * Apply any pending member additions or removals
-   *
-   * mutates members_ and clears members_to_modify_
+   * Get the members to write to storage, after accounting for duplicate members
+   * and member removals.
    */
-  virtual void apply_pending_changes() = 0;
+  virtual std::vector<std::shared_ptr<GroupMember>> members_to_serialize()
+      const = 0;
 
  protected:
   /* ********************************* */
@@ -198,6 +230,9 @@ class GroupDetails {
   /** The group URI. */
   URI group_uri_;
 
+  /** Whether the group has been modified. */
+  bool is_modified_;
+
   /**
    * The mapping of all members of this group. This is the canonical store of
    * the group's members. The key is the member's key().
@@ -232,9 +267,6 @@ class GroupDetails {
   /** Format version. */
   const uint32_t version_;
 
-  /** Were changes applied and is a write is required */
-  bool changes_applied_;
-
   /* ********************************* */
   /*         PROTECTED METHODS         */
   /* ********************************* */
@@ -248,7 +280,7 @@ class GroupDetails {
   /** Invalidate the built lookup tables. */
   void invalidate_lookups();
 };
-}  // namespace sm
-}  // namespace tiledb
+
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_GROUP_DETAILS_H
