@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2022 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -57,8 +57,7 @@ using namespace tiledb;
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class ReaderStatusException : public StatusException {
  public:
@@ -278,7 +277,7 @@ Status Reader::load_initial_data() {
 
   // Load delete conditions.
   auto&& [st, conditions, update_values] =
-      storage_manager_->load_delete_and_update_conditions(*array_);
+      array_->load_delete_and_update_conditions();
   RETURN_CANCEL_OR_ERROR(st);
   delete_and_update_conditions_ = std::move(*conditions);
 
@@ -353,17 +352,19 @@ Status Reader::apply_query_condition(
   if (stride == UINT64_MAX)
     stride = 1;
 
+  QueryCondition::Params params(query_memory_tracker_, array_schema_);
   if (condition_.has_value()) {
     RETURN_NOT_OK(condition_->apply(
-        array_schema_, fragment_metadata_, result_cell_slabs, stride));
+        params, fragment_metadata_, result_cell_slabs, stride));
   }
 
   // Apply delete conditions.
   if (!delete_and_update_conditions_.empty()) {
     for (uint64_t i = 0; i < delete_and_update_conditions_.size(); i++) {
       // For legacy, always run the timestamped condition.
+
       RETURN_NOT_OK(timestamped_delete_and_update_conditions_[i].apply(
-          array_schema_, fragment_metadata_, result_cell_slabs, stride));
+          params, fragment_metadata_, result_cell_slabs, stride));
     }
   }
 
@@ -371,7 +372,7 @@ Status Reader::apply_query_condition(
   if (!delete_timestamps_condition_.empty()) {
     // Remove cells with partial overlap from the bitmap.
     RETURN_NOT_OK(delete_timestamps_condition_.apply(
-        array_schema_, fragment_metadata_, result_cell_slabs, stride));
+        params, fragment_metadata_, result_cell_slabs, stride));
   }
 
   return Status::Ok();
@@ -454,7 +455,7 @@ Status Reader::compute_range_result_coords(
     }
   } else {  // Sparse
     auto resource =
-        query_memory_tracker_->get_resource(MemoryType::TILE_BITMAP);
+        query_memory_tracker_->get_resource(MemoryType::RESULT_TILE_BITMAP);
     tdb::pmr::vector<uint8_t> result_bitmap(coords_num, 1, resource);
 
     // Compute result and overwritten bitmap per dimension
@@ -476,8 +477,10 @@ Status Reader::compute_range_result_coords(
     // Apply partial overlap condition, if required.
     const auto frag_meta = fragment_metadata_[tile->frag_idx()];
     if (process_partial_timestamps(*frag_meta)) {
+      QueryCondition::Params params(
+          query_memory_tracker_, *(frag_meta->array_schema().get()));
       RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<uint8_t>(
-          *(frag_meta->array_schema().get()), *tile, result_bitmap));
+          params, *tile, result_bitmap));
     }
 
     // Gather results
@@ -515,8 +518,8 @@ Status Reader::compute_range_result_coords(
     }
   }
 
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, range_num, [&](uint64_t r) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, range_num, [&](uint64_t r) {
         // Compute overlapping coordinates per range
         RETURN_NOT_OK(compute_range_result_coords(
             subarray,
@@ -613,8 +616,8 @@ Status Reader::compute_range_result_coords(
   // Gather result range coordinates per fragment
   auto fragment_num = fragment_metadata_.size();
   std::vector<std::vector<ResultCoords>> range_result_coords_vec(fragment_num);
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, fragment_num, [&](uint32_t f) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, fragment_num, [&](uint32_t f) {
         return compute_range_result_coords(
             subarray,
             range_idx,
@@ -910,7 +913,7 @@ Status Reader::copy_fixed_cells(
       &cs_offsets,
       fixed_cs_partitions);
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       fixed_cs_partitions->size(),
       std::move(copy_fn));
@@ -934,10 +937,8 @@ void Reader::compute_fixed_cs_partitions(
     return;
   }
 
-  const auto num_copy_threads =
-      storage_manager_->compute_tp()->concurrency_level();
-
   // Calculate the partition sizes.
+  const auto num_copy_threads = resources_.compute_tp().concurrency_level();
   auto num_cs = result_cell_slabs.size();
   const uint64_t num_cs_partitions =
       std::min<uint64_t>(num_copy_threads, num_cs);
@@ -1102,7 +1103,7 @@ Status Reader::copy_var_cells(
       &var_offsets_per_cs,
       var_cs_partitions);
   auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, var_cs_partitions->size(), copy_fn);
+      &resources_.compute_tp(), 0, var_cs_partitions->size(), copy_fn);
 
   RETURN_NOT_OK(status);
 
@@ -1123,10 +1124,8 @@ void Reader::compute_var_cs_partitions(
     return;
   }
 
-  const auto num_copy_threads =
-      storage_manager_->compute_tp()->concurrency_level();
-
   // Calculate the partition range.
+  const auto num_copy_threads = resources_.compute_tp().concurrency_level();
   const uint64_t num_cs = result_cell_slabs.size();
   const uint64_t num_cs_partitions =
       std::min<uint64_t>(num_copy_threads, num_cs);
@@ -1571,7 +1570,7 @@ Status Reader::compute_result_cell_slabs_global(
     tile_subarrays.emplace_back(
         subarray.crop_to_tile((const T*)&tc[0], cell_order));
     auto& tile_subarray = tile_subarrays.back();
-    throw_if_not_ok(tile_subarray.template compute_tile_coords<T>());
+    tile_subarray.template compute_tile_coords<T>();
 
     RETURN_NOT_OK(compute_result_cell_slabs_row_col<T>(
         tile_subarray,
@@ -1726,7 +1725,7 @@ Status Reader::dense_read() {
   // `sparse_result_tiles` will hold all the relevant result tiles of
   // sparse fragments
   std::vector<ResultCoords> result_coords;
-  IndexedList<ResultTile> sparse_result_tiles;
+  IndexedList<ResultTile> sparse_result_tiles(query_memory_tracker_);
   RETURN_NOT_OK(compute_result_coords(sparse_result_tiles, result_coords));
 
   // Compute result cell slabs.
@@ -1738,7 +1737,7 @@ Status Reader::dense_read() {
   std::vector<ResultTile*> result_tiles;
   auto& subarray = read_state_.partitioner_.current();
 
-  RETURN_NOT_OK(subarray.compute_tile_coords<T>());
+  subarray.compute_tile_coords<T>();
   RETURN_NOT_OK(compute_result_cell_slabs<T>(
       subarray,
       result_space_tiles,
@@ -1788,10 +1787,11 @@ Status Reader::get_all_result_coords(
   if (fragment_metadata_[tile->frag_idx()]->has_timestamps() &&
       partial_overlap) {
     auto resource =
-        query_memory_tracker_->get_resource(MemoryType::TILE_BITMAP);
+        query_memory_tracker_->get_resource(MemoryType::RESULT_TILE_BITMAP);
     tdb::pmr::vector<uint8_t> result_bitmap(coords_num, 1, resource);
+    QueryCondition::Params params(query_memory_tracker_, array_schema_);
     RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<uint8_t>(
-        *(frag_meta->array_schema().get()), *tile, result_bitmap));
+        params, *tile, result_bitmap));
 
     for (uint64_t i = 0; i < coords_num; ++i) {
       if (result_bitmap[i]) {
@@ -1884,7 +1884,7 @@ void Reader::init_read_state() {
       memory_budget,
       memory_budget_var,
       memory_budget_validity,
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       stats_,
       logger_);
   read_state_.overflowed_ = false;
@@ -1948,26 +1948,23 @@ Status Reader::sort_result_coords(
 
   if (layout == Layout::ROW_MAJOR) {
     parallel_sort(
-        storage_manager_->compute_tp(), iter_begin, iter_end, RowCmp(domain));
+        &resources_.compute_tp(), iter_begin, iter_end, RowCmp(domain));
   } else if (layout == Layout::COL_MAJOR) {
     parallel_sort(
-        storage_manager_->compute_tp(), iter_begin, iter_end, ColCmp(domain));
+        &resources_.compute_tp(), iter_begin, iter_end, ColCmp(domain));
   } else if (layout == Layout::GLOBAL_ORDER) {
     if (array_schema_.cell_order() == Layout::HILBERT) {
       std::vector<std::pair<uint64_t, uint64_t>> hilbert_values(coords_num);
       RETURN_NOT_OK(calculate_hilbert_values(iter_begin, &hilbert_values));
       parallel_sort(
-          storage_manager_->compute_tp(),
+          &resources_.compute_tp(),
           hilbert_values.begin(),
           hilbert_values.end(),
           HilbertCmpRCI(domain, iter_begin));
       RETURN_NOT_OK(reorganize_result_coords(iter_begin, &hilbert_values));
     } else {
       parallel_sort(
-          storage_manager_->compute_tp(),
-          iter_begin,
-          iter_end,
-          GlobalCmp(domain));
+          &resources_.compute_tp(), iter_begin, iter_end, GlobalCmp(domain));
     }
   } else {
     assert(false);
@@ -1984,7 +1981,7 @@ Status Reader::sparse_read() {
   // `sparse_result_tiles` will hold all the relevant result tiles of
   // sparse fragments
   std::vector<ResultCoords> result_coords;
-  IndexedList<ResultTile> sparse_result_tiles;
+  IndexedList<ResultTile> sparse_result_tiles(query_memory_tracker_);
 
   RETURN_NOT_OK(compute_result_coords(sparse_result_tiles, result_coords));
   std::vector<ResultTile*> result_tiles;
@@ -2108,8 +2105,8 @@ Status Reader::calculate_hilbert_values(
   auto coords_num = (uint64_t)hilbert_values->size();
 
   // Calculate Hilbert values in parallel
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, coords_num, [&](uint64_t c) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, coords_num, [&](uint64_t c) {
         std::vector<uint64_t> coords(dim_num);
         for (uint32_t d = 0; d < dim_num; ++d) {
           auto dim{array_schema_.dimension_ptr(d)};
@@ -2394,5 +2391,4 @@ void Reader::fill_dense_coords_col_slab(
   }
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
