@@ -32,6 +32,7 @@
 
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_condition_combination_op.h"
 #include "tiledb/sm/enums/query_condition_op.h"
@@ -616,7 +617,7 @@ void QueryCondition::apply_ast_node(
     const ByteVecValue& fill_value,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
   const std::string& field_name = node->get_field_name();
   const void* condition_value_content = node->get_value_ptr();
   const size_t condition_value_size = node->get_value_size();
@@ -645,8 +646,10 @@ void QueryCondition::apply_ast_node(
       // delete condition was already processed, GT condition is always true.
       if (field_name == constants::delete_timestamps &&
           (!fragment_metadata[f]->has_delete_meta() ||
-           fragment_metadata[f]->get_processed_conditions_set().count(
-               condition_marker_) != 0)) {
+           fragment_metadata[f]
+                   ->loaded_metadata()
+                   ->get_processed_conditions_set()
+                   .count(condition_marker_) != 0)) {
         assert(Op == QueryConditionOp::GT);
         for (size_t c = starting_index; c < starting_index + length; ++c) {
           result_cell_bitmap[c] = 1;
@@ -768,7 +771,7 @@ void QueryCondition::apply_ast_node(
     const ByteVecValue& fill_value,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
   switch (node->get_op()) {
     case QueryConditionOp::ALWAYS_TRUE:
       apply_ast_node<T, QueryConditionOp::ALWAYS_TRUE, CombinationOp>(
@@ -907,7 +910,7 @@ void QueryCondition::apply_ast_node(
     const uint64_t stride,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
   std::string node_field_name = node->get_field_name();
 
   const auto nullable = array_schema.is_nullable(node_field_name);
@@ -1151,12 +1154,13 @@ void QueryCondition::apply_ast_node(
 template <typename CombinationOp>
 void QueryCondition::apply_tree(
     const tdb_unique_ptr<ASTNode>& node,
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     uint64_t stride,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
-    std::vector<uint8_t>& result_cell_bitmap) const {
+    tdb::pmr::vector<uint8_t>& result_cell_bitmap) const {
+  const auto& array_schema = params.GetSchema();
   if (!node->is_expr()) {
     apply_ast_node(
         node,
@@ -1191,7 +1195,7 @@ void QueryCondition::apply_tree(
           for (const auto& child : node->get_children()) {
             apply_tree(
                 child,
-                array_schema,
+                params,
                 fragment_metadata,
                 stride,
                 result_cell_slabs,
@@ -1203,12 +1207,15 @@ void QueryCondition::apply_tree(
         } else if constexpr (std::is_same_v<
                                  CombinationOp,
                                  std::logical_or<uint8_t>>) {
-          std::vector<uint8_t> combination_op_bitmap(result_bitmap_size, 1);
+          auto resource = params.GetMemoryTracker()->get_resource(
+              MemoryType::QUERY_CONDITION);
+          tdb::pmr::vector<uint8_t> combination_op_bitmap(
+              result_bitmap_size, 1, resource);
 
           for (const auto& child : node->get_children()) {
             apply_tree(
                 child,
-                array_schema,
+                params,
                 fragment_metadata,
                 stride,
                 result_cell_slabs,
@@ -1227,12 +1234,15 @@ void QueryCondition::apply_tree(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        std::vector<uint8_t> combination_op_bitmap(result_bitmap_size, 0);
+        auto resource = params.GetMemoryTracker()->get_resource(
+            MemoryType::QUERY_CONDITION);
+        tdb::pmr::vector<uint8_t> combination_op_bitmap(
+            result_bitmap_size, 0, resource);
 
         for (const auto& child : node->get_children()) {
           apply_tree(
               child,
-              array_schema,
+              params,
               fragment_metadata,
               stride,
               result_cell_slabs,
@@ -1257,7 +1267,7 @@ void QueryCondition::apply_tree(
 }
 
 Status QueryCondition::apply(
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     std::vector<ResultCellSlab>& result_cell_slabs,
     const uint64_t stride) const {
@@ -1270,10 +1280,13 @@ Status QueryCondition::apply(
     total_lengths += elem.length_;
   }
 
-  std::vector<uint8_t> result_cell_bitmap(total_lengths, 1);
+  auto resource =
+      params.GetMemoryTracker()->get_resource(MemoryType::QUERY_CONDITION);
+  tdb::pmr::vector<uint8_t> result_cell_bitmap(total_lengths, 1, resource);
+
   apply_tree(
       tree_,
-      array_schema,
+      params,
       fragment_metadata,
       stride,
       result_cell_slabs,
@@ -1934,7 +1947,7 @@ void QueryCondition::apply_ast_node_dense(
 template <typename CombinationOp>
 void QueryCondition::apply_tree_dense(
     const tdb_unique_ptr<ASTNode>& node,
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile* result_tile,
     const uint64_t start,
     const uint64_t src_cell,
@@ -1945,7 +1958,7 @@ void QueryCondition::apply_tree_dense(
   if (!node->is_expr()) {
     apply_ast_node_dense(
         node,
-        array_schema,
+        params.GetSchema(),
         result_tile,
         start,
         src_cell,
@@ -1978,7 +1991,7 @@ void QueryCondition::apply_tree_dense(
           for (const auto& child : node->get_children()) {
             apply_tree_dense(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 start,
                 src_cell,
@@ -1992,14 +2005,17 @@ void QueryCondition::apply_tree_dense(
         } else if constexpr (std::is_same_v<
                                  CombinationOp,
                                  std::logical_or<uint8_t>>) {
-          std::vector<uint8_t> combination_op_bitmap(result_buffer_size, 1);
+          auto resource = params.GetMemoryTracker()->get_resource(
+              MemoryType::QUERY_CONDITION);
+          tdb::pmr::vector<uint8_t> combination_op_bitmap(
+              result_buffer_size, 1, resource);
           span<uint8_t> combination_op_span(
               combination_op_bitmap.data(), result_buffer_size);
 
           for (const auto& child : node->get_children()) {
             apply_tree_dense(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 start,
                 src_cell,
@@ -2019,13 +2035,16 @@ void QueryCondition::apply_tree_dense(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        std::vector<uint8_t> combination_op_bitmap(result_buffer_size, 0);
+        auto resource = params.GetMemoryTracker()->get_resource(
+            MemoryType::QUERY_CONDITION);
+        tdb::pmr::vector<uint8_t> combination_op_bitmap(
+            result_buffer_size, 0, resource);
         span<uint8_t> combination_op_span(
             combination_op_bitmap.data(), result_buffer_size);
         for (const auto& child : node->get_children()) {
           apply_tree_dense(
               child,
-              array_schema,
+              params,
               result_tile,
               start,
               src_cell,
@@ -2052,7 +2071,7 @@ void QueryCondition::apply_tree_dense(
 }
 
 Status QueryCondition::apply_dense(
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile* result_tile,
     const uint64_t start,
     const uint64_t length,
@@ -2068,7 +2087,7 @@ Status QueryCondition::apply_dense(
   span<uint8_t> result_span(result_buffer + start, length);
   apply_tree_dense(
       tree_,
-      array_schema,
+      params,
       result_tile,
       start,
       src_cell,
@@ -2741,10 +2760,11 @@ void QueryCondition::apply_ast_node_sparse(
 template <typename BitmapType, typename CombinationOp>
 void QueryCondition::apply_tree_sparse(
     const tdb_unique_ptr<ASTNode>& node,
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile& result_tile,
     CombinationOp combination_op,
     tdb::pmr::vector<BitmapType>& result_bitmap) const {
+  const auto& array_schema = params.GetSchema();
   if (!node->is_expr()) {
     apply_ast_node_sparse<BitmapType>(
         node, array_schema, result_tile, combination_op, result_bitmap);
@@ -2774,7 +2794,7 @@ void QueryCondition::apply_tree_sparse(
           for (const auto& child : node->get_children()) {
             apply_tree_sparse<BitmapType>(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 std::multiplies<BitmapType>(),
                 result_bitmap);
@@ -2783,14 +2803,15 @@ void QueryCondition::apply_tree_sparse(
           // Handle the cl'(q, a) case.
           // This cases on whether the combination op = OR.
         } else if constexpr (std::is_same_v<CombinationOp, QCMax<BitmapType>>) {
-          auto resource = result_bitmap.get_allocator().resource();
+          auto resource = params.GetMemoryTracker()->get_resource(
+              MemoryType::QUERY_CONDITION);
           tdb::pmr::vector<BitmapType> combination_op_bitmap(
               result_bitmap_size, 1, resource);
 
           for (const auto& child : node->get_children()) {
             apply_tree_sparse<BitmapType>(
                 child,
-                array_schema,
+                params,
                 result_tile,
                 std::multiplies<BitmapType>(),
                 combination_op_bitmap);
@@ -2807,14 +2828,15 @@ void QueryCondition::apply_tree_sparse(
          *                        = a /\ (cl1'(q; cl2'(q; 0)))
          */
       case QueryConditionCombinationOp::OR: {
-        auto resource = result_bitmap.get_allocator().resource();
+        auto resource = params.GetMemoryTracker()->get_resource(
+            MemoryType::QUERY_CONDITION);
         tdb::pmr::vector<BitmapType> combination_op_bitmap(
             result_bitmap_size, 0, resource);
 
         for (const auto& child : node->get_children()) {
           apply_tree_sparse<BitmapType>(
               child,
-              array_schema,
+              params,
               result_tile,
               QCMax<BitmapType>(),
               combination_op_bitmap);
@@ -2838,15 +2860,11 @@ void QueryCondition::apply_tree_sparse(
 
 template <typename BitmapType>
 Status QueryCondition::apply_sparse(
-    const ArraySchema& array_schema,
+    const QueryCondition::Params& params,
     ResultTile& result_tile,
     tdb::pmr::vector<BitmapType>& result_bitmap) {
   apply_tree_sparse<BitmapType>(
-      tree_,
-      array_schema,
-      result_tile,
-      std::multiplies<BitmapType>(),
-      result_bitmap);
+      tree_, params, result_tile, std::multiplies<BitmapType>(), result_bitmap);
 
   return Status::Ok();
 }
@@ -2869,7 +2887,7 @@ uint64_t QueryCondition::condition_index() const {
 
 // Explicit template instantiations.
 template Status QueryCondition::apply_sparse<uint8_t>(
-    const ArraySchema& array_schema, ResultTile&, tdb::pmr::vector<uint8_t>&);
+    const QueryCondition::Params&, ResultTile&, tdb::pmr::vector<uint8_t>&);
 template Status QueryCondition::apply_sparse<uint64_t>(
-    const ArraySchema& array_schema, ResultTile&, tdb::pmr::vector<uint64_t>&);
+    const QueryCondition::Params&, ResultTile&, tdb::pmr::vector<uint64_t>&);
 }  // namespace tiledb::sm
