@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -150,7 +150,7 @@ std::string OrderedWriter::name() {
 
 void OrderedWriter::clean_up() {
   if (frag_uri_.has_value()) {
-    throw_if_not_ok(storage_manager_->vfs()->remove_dir(frag_uri_.value()));
+    throw_if_not_ok(resources_.vfs().remove_dir(frag_uri_.value()));
   }
 }
 
@@ -196,14 +196,14 @@ Status OrderedWriter::ordered_write() {
 
   // Prepare, filter and write tiles for all attributes
   auto attr_num = buffers_.size();
-  auto compute_tp = storage_manager_->compute_tp();
+  auto* compute_tp = &resources_.compute_tp();
   auto thread_num = compute_tp->concurrency_level();
   std::unordered_map<std::string, IndexedList<WriterTileTupleVector>> tiles;
   for (const auto& buff : buffers_) {
     tiles.emplace(
         std::piecewise_construct,
         std::forward_as_tuple(buff.first),
-        std::forward_as_tuple());
+        std::forward_as_tuple(query_memory_tracker_));
   }
 
   if (attr_num > tile_num) {  // Parallelize over attributes
@@ -211,14 +211,14 @@ Status OrderedWriter::ordered_write() {
       auto buff_it = buffers_.begin();
       std::advance(buff_it, i);
       const auto& attr = buff_it->first;
-      auto& attr_tile_batches = tiles[attr];
+      auto& attr_tile_batches = tiles.at(attr);
       return prepare_filter_and_write_tiles<T>(
           attr, attr_tile_batches, frag_meta, &dense_tiler, 1);
     }));
   } else {  // Parallelize over tiles
     for (const auto& buff : buffers_) {
       const auto& attr = buff.first;
-      auto& attr_tile_batches = tiles[attr];
+      auto& attr_tile_batches = tiles.at(attr);
       RETURN_NOT_OK(prepare_filter_and_write_tiles<T>(
           attr, attr_tile_batches, frag_meta, &dense_tiler, thread_num));
     }
@@ -233,7 +233,7 @@ Status OrderedWriter::ordered_write() {
       const auto var_size = array_schema_.var_size(attr);
       if (has_min_max_metadata(attr, var_size) &&
           array_schema_.var_size(attr)) {
-        auto& attr_tile_batches = tiles[attr];
+        auto& attr_tile_batches = tiles.at(attr);
         frag_meta->convert_tile_min_max_var_sizes_to_offsets(attr);
         for (auto& batch : attr_tile_batches) {
           uint64_t idx = 0;
@@ -249,7 +249,7 @@ Status OrderedWriter::ordered_write() {
   } else {  // Parallelize over tiles
     for (const auto& buff : buffers_) {
       const auto& attr = buff.first;
-      auto& attr_tile_batches = tiles[attr];
+      auto& attr_tile_batches = tiles.at(attr);
       const auto var_size = array_schema_.var_size(attr);
       if (has_min_max_metadata(attr, var_size) &&
           array_schema_.var_size(attr)) {
@@ -257,7 +257,7 @@ Status OrderedWriter::ordered_write() {
         RETURN_NOT_OK(parallel_for(
             compute_tp, 0, attr_tile_batches.size(), [&](uint64_t b) {
               const auto& attr = buff.first;
-              auto& batch = tiles[attr][b];
+              auto& batch = tiles.at(attr)[b];
               auto idx = b * thread_num;
               for (auto& tile : batch) {
                 frag_meta->set_tile_min_var(attr, idx, tile.min());
@@ -279,7 +279,7 @@ Status OrderedWriter::ordered_write() {
 
   // The following will make the fragment visible
   URI commit_uri = array_->array_directory().get_commit_uri(frag_uri_.value());
-  RETURN_NOT_OK(storage_manager_->vfs()->touch(commit_uri));
+  throw_if_not_ok(resources_.vfs().touch(commit_uri));
 
   return Status::Ok();
 }
@@ -334,7 +334,7 @@ Status OrderedWriter::prepare_filter_and_write_tiles(
     {
       auto timer_se = stats_->start_timer("prepare_and_filter_tiles");
       auto st = parallel_for(
-          storage_manager_->compute_tp(), 0, batch_size, [&](uint64_t i) {
+          &resources_.compute_tp(), 0, batch_size, [&](uint64_t i) {
             // Prepare and filter tiles
             auto& writer_tile = tile_batches[b][i];
             RETURN_NOT_OK(
@@ -364,7 +364,7 @@ Status OrderedWriter::prepare_filter_and_write_tiles(
       RETURN_NOT_OK(write_task->get());
     }
 
-    write_task = storage_manager_->io_tp()->execute([&, b, frag_tile_id]() {
+    write_task = resources_.io_tp().execute([&, b, frag_tile_id]() {
       close_files = (b == batch_num - 1);
       RETURN_NOT_OK(write_tiles(
           0,
