@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -89,6 +89,7 @@ UnorderedWriter::UnorderedWriter(
           remote_query,
           fragment_name)
     , frag_uri_(std::nullopt)
+    , cell_pos_(query_memory_tracker_->get_resource(MemoryType::WRITER_DATA))
     , written_buffers_(written_buffers)
     , is_coords_pass_(true) {
   // Check the layout is unordered.
@@ -180,7 +181,7 @@ Status UnorderedWriter::alloc_frag_meta() {
 
 void UnorderedWriter::clean_up() {
   if (frag_uri_.has_value()) {
-    throw_if_not_ok(storage_manager_->vfs()->remove_dir(frag_uri_.value()));
+    throw_if_not_ok(resources_.vfs().remove_dir(frag_uri_.value()));
   }
 }
 
@@ -216,10 +217,7 @@ Status UnorderedWriter::check_coord_dups() const {
   }
 
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -309,10 +307,7 @@ Status UnorderedWriter::compute_coord_dups() {
 
   std::mutex mtx;
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -372,7 +367,7 @@ Status UnorderedWriter::compute_coord_dups() {
 }
 
 Status UnorderedWriter::prepare_tiles(
-    std::unordered_map<std::string, WriterTileTupleVector>* tiles) const {
+    tdb::pmr::unordered_map<std::string, WriterTileTupleVector>* tiles) const {
   auto timer_se = stats_->start_timer("prepare_tiles");
 
   // Initialize attribute tiles
@@ -380,16 +375,16 @@ Status UnorderedWriter::prepare_tiles(
   for (const auto& it : buffers_) {
     const auto& name = it.first;
     if (written_buffers_.count(name) == 0) {
-      (*tiles).emplace(
+      tiles->emplace(
           std::piecewise_construct,
           std::forward_as_tuple(name),
-          std::forward_as_tuple());
+          std::forward_as_tuple(query_memory_tracker_));
     }
   }
 
   // Prepare tiles for all attributes and coordinates
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, tiles->size(), [&](uint64_t i) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, tiles->size(), [&](uint64_t i) {
         auto tiles_it = tiles->begin();
         std::advance(tiles_it, i);
         RETURN_CANCEL_OR_ERROR(
@@ -620,7 +615,7 @@ Status UnorderedWriter::sort_coords() {
   DomainBuffersView domain_buffs{array_schema_, buffers_};
   if (cell_order != Layout::HILBERT) {  // Row- or col-major
     parallel_sort(
-        storage_manager_->compute_tp(),
+        &resources_.compute_tp(),
         cell_pos_.begin(),
         cell_pos_.end(),
         GlobalCmpQB(domain, domain_buffs));
@@ -628,7 +623,7 @@ Status UnorderedWriter::sort_coords() {
     std::vector<uint64_t> hilbert_values(coords_info_.coords_num_);
     RETURN_NOT_OK(calculate_hilbert_values(domain_buffs, hilbert_values));
     parallel_sort(
-        storage_manager_->compute_tp(),
+        &resources_.compute_tp(),
         cell_pos_.begin(),
         cell_pos_.end(),
         HilbertCmpQB(domain, domain_buffs, hilbert_values));
@@ -675,7 +670,8 @@ Status UnorderedWriter::unordered_write() {
   frag_uri_ = frag_meta_->fragment_uri();
 
   // Prepare tiles
-  std::unordered_map<std::string, WriterTileTupleVector> tiles;
+  tdb::pmr::unordered_map<std::string, WriterTileTupleVector> tiles(
+      query_memory_tracker_->get_resource(MemoryType::WRITER_TILE_DATA));
   RETURN_CANCEL_OR_ERROR(prepare_tiles(&tiles));
 
   // No tiles
@@ -730,8 +726,7 @@ Status UnorderedWriter::unordered_write() {
     // The following will make the fragment visible
     URI commit_uri =
         array_->array_directory().get_commit_uri(frag_uri_.value());
-
-    RETURN_NOT_OK(storage_manager_->vfs()->touch(commit_uri));
+    throw_if_not_ok(resources_.vfs().touch(commit_uri));
 
     // Clear some data to prevent it from being serialized.
     cell_pos_.clear();
