@@ -311,7 +311,10 @@ void WriterBase::refresh_config() {
 
 shared_ptr<FragmentMetadata> WriterBase::create_fragment_metadata() {
   return make_shared<FragmentMetadata>(
-      HERE(), &storage_manager_->resources(), query_memory_tracker_);
+      HERE(),
+      &resources_,
+      query_memory_tracker_,
+      array_->array_schema_latest().write_version());
 }
 
 Status WriterBase::add_written_fragment_info(const URI& uri) {
@@ -330,10 +333,7 @@ Status WriterBase::calculate_hilbert_values(
   // Calculate Hilbert values in parallel
   assert(hilbert_values.size() >= coords_info_.coords_num_);
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      coords_info_.coords_num_,
-      [&](uint64_t c) {
+      &resources_.compute_tp(), 0, coords_info_.coords_num_, [&](uint64_t c) {
         std::vector<uint64_t> coords(dim_num);
         for (uint32_t d = 0; d < dim_num; ++d) {
           auto dim{array_schema_.dimension_ptr(d)};
@@ -424,7 +424,7 @@ Status WriterBase::check_coord_oob() const {
 
   // Check if all coordinates fall in the domain in parallel
   auto status = parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       coords_info_.coords_num_,
       0,
@@ -606,18 +606,18 @@ Status WriterBase::close_files(shared_ptr<FragmentMetadata> meta) const {
     }
   }
 
-  auto status = parallel_for(
-      storage_manager_->io_tp(), 0, file_uris.size(), [&](uint64_t i) {
+  auto status =
+      parallel_for(&resources_.io_tp(), 0, file_uris.size(), [&](uint64_t i) {
         const auto& file_uri = file_uris[i];
         if (layout_ == Layout::GLOBAL_ORDER && remote_query()) {
-          storage_manager_->vfs()->finalize_and_close_file(file_uri);
+          resources_.vfs().finalize_and_close_file(file_uri);
         } else {
-          RETURN_NOT_OK(storage_manager_->vfs()->close_file(file_uri));
+          throw_if_not_ok(resources_.vfs().close_file(file_uri));
         }
         return Status::Ok();
       });
 
-  RETURN_NOT_OK(status);
+  throw_if_not_ok(status);
 
   return Status::Ok();
 }
@@ -644,8 +644,8 @@ std::vector<NDRange> WriterBase::compute_mbrs(
 
   // Compute MBRs
   std::vector<NDRange> mbrs(tile_num);
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, tile_num, [&](uint64_t i) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, tile_num, [&](uint64_t i) {
         mbrs[i].resize(dim_num);
         std::vector<const void*> data(dim_num);
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -684,10 +684,7 @@ void WriterBase::set_coords_metadata(
   }
 
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      start_tile_idx,
-      end_tile_idx,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), start_tile_idx, end_tile_idx, [&](uint64_t i) {
         meta->set_mbr(i - start_tile_idx, mbrs[i]);
         return Status::Ok();
       });
@@ -703,7 +700,7 @@ void WriterBase::set_coords_metadata(
 Status WriterBase::compute_tiles_metadata(
     uint64_t tile_num,
     tdb::pmr::unordered_map<std::string, WriterTileTupleVector>& tiles) const {
-  auto compute_tp = storage_manager_->compute_tp();
+  auto* compute_tp = &resources_.compute_tp();
 
   // Parallelize over attributes?
   if (tiles.size() > tile_num) {
@@ -779,11 +776,11 @@ Status WriterBase::create_fragment(
   // Create the directories.
   // Create the fragment directory, the directory for the new fragment
   // URI, and the commit directory.
-  throw_if_not_ok(storage_manager_->vfs()->create_dir(
-      array_dir.get_fragments_dir(write_version)));
-  throw_if_not_ok(storage_manager_->vfs()->create_dir(fragment_uri_));
-  throw_if_not_ok(storage_manager_->vfs()->create_dir(
-      array_dir.get_commits_dir(write_version)));
+  throw_if_not_ok(
+      resources_.vfs().create_dir(array_dir.get_fragments_dir(write_version)));
+  throw_if_not_ok(resources_.vfs().create_dir(fragment_uri_));
+  throw_if_not_ok(
+      resources_.vfs().create_dir(array_dir.get_commits_dir(write_version)));
 
   // Create fragment metadata.
   auto timestamp_range = std::pair<uint64_t, uint64_t>(timestamp, timestamp);
@@ -792,7 +789,7 @@ Status WriterBase::create_fragment(
       buffers_.count(constants::delete_timestamps) != 0;
   frag_meta = make_shared<FragmentMetadata>(
       HERE(),
-      &storage_manager_->resources(),
+      &resources_,
       array_->array_schema_latest_ptr(),
       fragment_uri_,
       timestamp_range,
@@ -808,8 +805,8 @@ Status WriterBase::create_fragment(
 Status WriterBase::filter_tiles(
     tdb::pmr::unordered_map<std::string, WriterTileTupleVector>* tiles) {
   auto timer_se = stats_->start_timer("filter_tiles");
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, tiles->size(), [&](uint64_t i) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, tiles->size(), [&](uint64_t i) {
         auto tiles_it = tiles->begin();
         std::advance(tiles_it, i);
         RETURN_CANCEL_OR_ERROR(
@@ -845,8 +842,8 @@ Status WriterBase::filter_tiles(
   }
 
   // For fixed size, process everything, for var size, everything minus offsets.
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, args.size(), [&](uint64_t i) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, args.size(), [&](uint64_t i) {
         const auto& [tile, offset_tile, contains_offsets, is_nullable] =
             args[i];
         RETURN_NOT_OK(filter_tile(
@@ -858,7 +855,7 @@ Status WriterBase::filter_tiles(
   // Process offsets for var size.
   if (var_size) {
     auto status = parallel_for(
-        storage_manager_->compute_tp(), 0, tiles->size(), [&](uint64_t i) {
+        &resources_.compute_tp(), 0, tiles->size(), [&](uint64_t i) {
           auto& tile = (*tiles)[i];
           RETURN_NOT_OK(
               filter_tile(name, &tile.offset_tile(), nullptr, true, false));
@@ -911,11 +908,7 @@ Status WriterBase::filter_tile(
 
   assert(!tile->filtered());
   RETURN_NOT_OK(filters.run_forward(
-      stats_,
-      tile,
-      offsets_tile,
-      storage_manager_->compute_tp(),
-      use_chunking));
+      stats_, tile, offsets_tile, &resources_.compute_tp(), use_chunking));
   assert(tile->filtered());
 
   return Status::Ok();
@@ -1056,7 +1049,7 @@ Status WriterBase::write_tiles(
 
   std::vector<ThreadPool::Task> tasks;
   for (auto& it : *tiles) {
-    tasks.push_back(storage_manager_->io_tp()->execute([&, this]() {
+    tasks.push_back(resources_.io_tp().execute([&, this]() {
       auto& attr = it.first;
       auto& tiles = it.second;
       RETURN_CANCEL_OR_ERROR(write_tiles(
@@ -1080,7 +1073,7 @@ Status WriterBase::write_tiles(
   }
 
   // Wait for writes and check all statuses
-  auto statuses = storage_manager_->io_tp()->wait_all_status(tasks);
+  auto statuses = resources_.io_tp().wait_all_status(tasks);
   for (auto& st : statuses)
     RETURN_NOT_OK(st);
 
@@ -1122,7 +1115,7 @@ Status WriterBase::write_tiles(
        ++i, ++tile_id) {
     auto& tile = (*tiles)[i];
     auto& t = var_size ? tile.offset_tile() : tile.fixed_tile();
-    RETURN_NOT_OK(storage_manager_->vfs()->write(
+    throw_if_not_ok(resources_.vfs().write(
         uri,
         t.filtered_buffer().data(),
         t.filtered_buffer().size(),
@@ -1132,7 +1125,7 @@ Status WriterBase::write_tiles(
 
     if (var_size) {
       auto& t_var = tile.var_tile();
-      RETURN_NOT_OK(storage_manager_->vfs()->write(
+      throw_if_not_ok(resources_.vfs().write(
           var_uri,
           t_var.filtered_buffer().data(),
           t_var.filtered_buffer().size(),
@@ -1157,7 +1150,7 @@ Status WriterBase::write_tiles(
 
     if (nullable) {
       auto& t_val = tile.validity_tile();
-      RETURN_NOT_OK(storage_manager_->vfs()->write(
+      throw_if_not_ok(resources_.vfs().write(
           validity_uri,
           t_val.filtered_buffer().data(),
           t_val.filtered_buffer().size(),
@@ -1186,11 +1179,10 @@ Status WriterBase::write_tiles(
         // requirement of remote global order writes, it should only be
         // done if this code is executed as a result of a remote query
         if (remote_query()) {
-          RETURN_NOT_OK(
-              storage_manager_->vfs()->flush_multipart_file_buffer(u));
+          throw_if_not_ok(resources_.vfs().flush_multipart_file_buffer(u));
         }
       } else {
-        RETURN_NOT_OK(storage_manager_->vfs()->close_file(u));
+        throw_if_not_ok(resources_.vfs().close_file(u));
       }
     }
   }
