@@ -131,7 +131,7 @@ void InfoCommand::print_tile_sizes() const {
   // Compute and report mean persisted tile sizes over all attributes.
   const auto& schema = array.array_schema_latest();
   auto fragment_metadata = array.fragment_metadata();
-  auto attributes = schema.attributes();
+  auto& attributes = schema.attributes();
   uint64_t total_persisted_size = 0, total_in_memory_size = 0;
 
   // Helper function for processing each attribute.
@@ -142,15 +142,18 @@ void InfoCommand::print_tile_sizes() const {
       uint64_t tile_num = f->tile_num();
       std::vector<std::string> names;
       names.push_back(name);
-      f->load_tile_offsets(enc_key, names);
-      f->load_tile_var_sizes(enc_key, name);
+      f->loaded_metadata()->load_tile_offsets(enc_key, names);
+      f->loaded_metadata()->load_tile_var_sizes(enc_key, name);
       for (uint64_t tile_idx = 0; tile_idx < tile_num; tile_idx++) {
-        persisted_tile_size += f->persisted_tile_size(name, tile_idx);
+        persisted_tile_size +=
+            f->loaded_metadata()->persisted_tile_size(name, tile_idx);
         in_memory_tile_size += f->tile_size(name, tile_idx);
         num_tiles++;
         if (var_size) {
-          persisted_tile_size += f->persisted_tile_var_size(name, tile_idx);
-          in_memory_tile_size += f->tile_var_size(name, tile_idx);
+          persisted_tile_size +=
+              f->loaded_metadata()->persisted_tile_var_size(name, tile_idx);
+          in_memory_tile_size +=
+              f->loaded_metadata()->tile_var_size(name, tile_idx);
           num_tiles++;
         }
       }
@@ -298,7 +301,7 @@ void InfoCommand::write_text_mbrs() const {
   auto fragment_metadata = array.fragment_metadata();
   std::stringstream text;
   for (const auto& f : fragment_metadata) {
-    f->load_rtree(*encryption_key);
+    f->loaded_metadata()->load_rtree(*encryption_key);
     const auto& mbrs = f->mbrs();
     for (const auto& mbr : mbrs) {
       auto str_mbr = mbr_to_string(mbr, schema.domain());
